/*
* Copyright (c) 2019, Nordic Semiconductor
* All rights reserved.
*
* Redistribution and use in source and binary forms, with or without modification,
* are permitted provided that the following conditions are met:
*
* 1. Redistributions of source code must retain the above copyright notice, this
*    list of conditions and the following disclaimer.
*
* 2. Redistributions in binary form must reproduce the above copyright notice, this
*    list of conditions and the following disclaimer in the documentation and/or
*    other materials provided with the distribution.
*
* 3. Neither the name of the copyright holder nor the names of its contributors may
*    be used to endorse or promote products derived from this software without
*    specific prior written permission.
*
* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
* ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
* WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
* IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
* INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
* WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
* ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
* POSSIBILITY OF SUCH DAMAGE.
*/

import Foundation

// Source: https://www.bluetooth.com/specifications/assigned-numbers/company-identifiers/
// Last update date: 19 August 2020
public struct CompanyIdentifier {
    
    private init() {
        // Forbidding instantiating.
    }
    
    public static func name(for id: UInt16) -> String? {
        switch id {
        case 0x0000: return "Ericsson Technology Licensing"
        case 0x0001: return "Nokia Mobile Phones"
        case 0x0002: return "Intel Corp."
        case 0x0003: return "IBM Corp."
        case 0x0004: return "Toshiba Corp."
        case 0x0005: return "3Com"
        case 0x0006: return "Microsoft"
        case 0x0007: return "Lucent"
        case 0x0008: return "Motorola"
        case 0x0009: return "Infineon Technologies AG"
        case 0x000A: return "Qualcomm Technologies International, Ltd. (QTIL)"
        case 0x000B: return "Silicon Wave"
        case 0x000C: return "Digianswer A/S"
        case 0x000D: return "Texas Instruments Inc."
        case 0x000E: return "Parthus Technologies Inc."
        case 0x000F: return "Broadcom Corporation"
        case 0x0010: return "Mitel Semiconductor"
        case 0x0011: return "Widcomm, Inc."
        case 0x0012: return "Zeevo, Inc."
        case 0x0013: return "Atmel Corporation"
        case 0x0014: return "Mitsubishi Electric Corporation"
        case 0x0015: return "RTX Telecom A/S"
        case 0x0016: return "KC Technology Inc."
        case 0x0017: return "Newlogic"
        case 0x0018: return "Transilica, Inc."
        case 0x0019: return "Rohde & Schwarz GmbH & Co. KG"
        case 0x001A: return "TTPCom Limited"
        case 0x001B: return "Signia Technologies, Inc."
        case 0x001C: return "Conexant Systems Inc."
        case 0x001D: return "Qualcomm"
        case 0x001E: return "Inventel"
        case 0x001F: return "AVM Berlin"
        case 0x0020: return "BandSpeed, Inc."
        case 0x0021: return "Mansella Ltd"
        case 0x0022: return "NEC Corporation"
        case 0x0023: return "WavePlus Technology Co., Ltd."
        case 0x0024: return "Alcatel"
        case 0x0025: return "NXP Semiconductors (formerly Philips Semiconductors)"
        case 0x0026: return "C Technologies"
        case 0x0027: return "Open Interface"
        case 0x0028: return "R F Micro Devices"
        case 0x0029: return "Hitachi Ltd"
        case 0x002A: return "Symbol Technologies, Inc."
        case 0x002B: return "Tenovis"
        case 0x002C: return "Macronix International Co. Ltd."
        case 0x002D: return "GCT Semiconductor"
        case 0x002E: return "Norwood Systems"
        case 0x002F: return "MewTel Technology Inc."
        case 0x0030: return "ST Microelectronics"
        case 0x0031: return "Synopsys, Inc."
        case 0x0032: return "Red-M (Communications) Ltd"
        case 0x0033: return "Commil Ltd"
        case 0x0034: return "Computer Access Technology Corporation (CATC)"
        case 0x0035: return "Eclipse (HQ Espana) S.L."
        case 0x0036: return "Renesas Electronics Corporation"
        case 0x0037: return "Mobilian Corporation"
        case 0x0038: return "Syntronix Corporation"
        case 0x0039: return "Integrated System Solution Corp."
        case 0x003A: return "Panasonic Corporation (formerly Matsushita Electric Industrial Co., Ltd.)"
        case 0x003B: return "Gennum Corporation"
        case 0x003C: return "BlackBerry Limited (formerly Research In Motion)"
        case 0x003D: return "IPextreme, Inc."
        case 0x003E: return "Systems and Chips, Inc"
        case 0x003F: return "Bluetooth SIG, Inc"
        case 0x0040: return "Seiko Epson Corporation"
        case 0x0041: return "Integrated Silicon Solution Taiwan, Inc."
        case 0x0042: return "CONWISE Technology Corporation Ltd"
        case 0x0043: return "PARROT AUTOMOTIVE SAS"
        case 0x0044: return "Socket Mobile"
        case 0x0045: return "Atheros Communications, Inc."
        case 0x0046: return "MediaTek, Inc."
        case 0x0047: return "Bluegiga"
        case 0x0048: return "Marvell Technology Group Ltd."
        case 0x0049: return "3DSP Corporation"
        case 0x004A: return "Accel Semiconductor Ltd."
        case 0x004B: return "Continental Automotive Systems"
        case 0x004C: return "Apple, Inc."
        case 0x004D: return "Staccato Communications, Inc."
        case 0x004E: return "Avago Technologies"
        case 0x004F: return "APT Ltd."
        case 0x0050: return "SiRF Technology, Inc."
        case 0x0051: return "Tzero Technologies, Inc."
        case 0x0052: return "J&M Corporation"
        case 0x0053: return "Free2move AB"
        case 0x0054: return "3DiJoy Corporation"
        case 0x0055: return "Plantronics, Inc."
        case 0x0056: return "Sony Ericsson Mobile Communications"
        case 0x0057: return "Harman International Industries, Inc."
        case 0x0058: return "Vizio, Inc."
        case 0x0059: return "Nordic Semiconductor ASA"
        case 0x005A: return "EM Microelectronic-Marin SA"
        case 0x005B: return "Ralink Technology Corporation"
        case 0x005C: return "Belkin International, Inc."
        case 0x005D: return "Realtek Semiconductor Corporation"
        case 0x005E: return "Stonestreet One, LLC"
        case 0x005F: return "Wicentric, Inc."
        case 0x0060: return "RivieraWaves S.A.S"
        case 0x0061: return "RDA Microelectronics"
        case 0x0062: return "Gibson Guitars"
        case 0x0063: return "MiCommand Inc."
        case 0x0064: return "Band XI International, LLC"
        case 0x0065: return "Hewlett-Packard Company"
        case 0x0066: return "9Solutions Oy"
        case 0x0067: return "GN Netcom A/S"
        case 0x0068: return "General Motors"
        case 0x0069: return "A&D Engineering, Inc."
        case 0x006A: return "MindTree Ltd."
        case 0x006B: return "Polar Electro OY"
        case 0x006C: return "Beautiful Enterprise Co., Ltd."
        case 0x006D: return "BriarTek, Inc"
        case 0x006E: return "Summit Data Communications, Inc."
        case 0x006F: return "Sound ID"
        case 0x0070: return "Monster, LLC"
        case 0x0071: return "connectBlue AB"
        case 0x0072: return "ShangHai Super Smart Electronics Co. Ltd."
        case 0x0073: return "Group Sense Ltd."
        case 0x0074: return "Zomm, LLC"
        case 0x0075: return "Samsung Electronics Co. Ltd."
        case 0x0076: return "Creative Technology Ltd."
        case 0x0077: return "Laird Technologies"
        case 0x0078: return "Nike, Inc."
        case 0x0079: return "lesswire AG"
        case 0x007A: return "MStar Semiconductor, Inc."
        case 0x007B: return "Hanlynn Technologies"
        case 0x007C: return "A & R Cambridge"
        case 0x007D: return "Seers Technology Co., Ltd."
        case 0x007E: return "Sports Tracking Technologies Ltd."
        case 0x007F: return "Autonet Mobile"
        case 0x0080: return "DeLorme Publishing Company, Inc."
        case 0x0081: return "WuXi Vimicro"
        case 0x0082: return "Sennheiser Communications A/S"
        case 0x0083: return "TimeKeeping Systems, Inc."
        case 0x0084: return "Ludus Helsinki Ltd."
        case 0x0085: return "BlueRadios, Inc."
        case 0x0086: return "Equinux AG"
        case 0x0087: return "Garmin International, Inc."
        case 0x0088: return "Ecotest"
        case 0x0089: return "GN ReSound A/S"
        case 0x008A: return "Jawbone"
        case 0x008B: return "Topcon Positioning Systems, LLC"
        case 0x008C: return "Gimbal Inc. (formerly Qualcomm Labs, Inc. and Qualcomm Retail Solutions, Inc.)"
        case 0x008D: return "Zscan Software"
        case 0x008E: return "Quintic Corp"
        case 0x008F: return "Telit Wireless Solutions GmbH (formerly Stollmann E+V GmbH)"
        case 0x0090: return "Funai Electric Co., Ltd."
        case 0x0091: return "Advanced PANMOBIL systems GmbH & Co. KG"
        case 0x0092: return "ThinkOptics, Inc."
        case 0x0093: return "Universal Electronics, Inc."
        case 0x0094: return "Airoha Technology Corp."
        case 0x0095: return "NEC Lighting, Ltd."
        case 0x0096: return "ODM Technology, Inc."
        case 0x0097: return "ConnecteDevice Ltd."
        case 0x0098: return "zero1.tv GmbH"
        case 0x0099: return "i.Tech Dynamic Global Distribution Ltd."
        case 0x009A: return "Alpwise"
        case 0x009B: return "Jiangsu Toppower Automotive Electronics Co., Ltd."
        case 0x009C: return "Colorfy, Inc."
        case 0x009D: return "Geoforce Inc."
        case 0x009E: return "Bose Corporation"
        case 0x009F: return "Suunto Oy"
        case 0x00A0: return "Kensington Computer Products Group"
        case 0x00A1: return "SR-Medizinelektronik"
        case 0x00A2: return "Vertu Corporation Limited"
        case 0x00A3: return "Meta Watch Ltd."
        case 0x00A4: return "LINAK A/S"
        case 0x00A5: return "OTL Dynamics LLC"
        case 0x00A6: return "Panda Ocean Inc."
        case 0x00A7: return "Visteon Corporation"
        case 0x00A8: return "ARP Devices Limited"
        case 0x00A9: return "Magneti Marelli S.p.A"
        case 0x00AA: return "CAEN RFID srl"
        case 0x00AB: return "Ingenieur-Systemgruppe Zahn GmbH"
        case 0x00AC: return "Green Throttle Games"
        case 0x00AD: return "Peter Systemtechnik GmbH"
        case 0x00AE: return "Omegawave Oy"
        case 0x00AF: return "Cinetix"
        case 0x00B0: return "Passif Semiconductor Corp"
        case 0x00B1: return "Saris Cycling Group, Inc"
        case 0x00B2: return "Bekey A/S"
        case 0x00B3: return "Clarinox Technologies Pty. Ltd."
        case 0x00B4: return "BDE Technology Co., Ltd."
        case 0x00B5: return "Swirl Networks"
        case 0x00B6: return "Meso international"
        case 0x00B7: return "TreLab Ltd"
        case 0x00B8: return "Qualcomm Innovation Center, Inc. (QuIC)"
        case 0x00B9: return "Johnson Controls, Inc."
        case 0x00BA: return "Starkey Laboratories Inc."
        case 0x00BB: return "S-Power Electronics Limited"
        case 0x00BC: return "Ace Sensor Inc"
        case 0x00BD: return "Aplix Corporation"
        case 0x00BE: return "AAMP of America"
        case 0x00BF: return "Stalmart Technology Limited"
        case 0x00C0: return "AMICCOM Electronics Corporation"
        case 0x00C1: return "Shenzhen Excelsecu Data Technology Co.,Ltd"
        case 0x00C2: return "Geneq Inc."
        case 0x00C3: return "Adidas AG"
        case 0x00C4: return "LG Electronics?"
        case 0x00C5: return "Onset Computer Corporation"
        case 0x00C6: return "Selfly BV"
        case 0x00C7: return "Quuppa Oy."
        case 0x00C8: return "GeLo Inc"
        case 0x00C9: return "Evluma"
        case 0x00CA: return "MC10"
        case 0x00CB: return "Binauric SE"
        case 0x00CC: return "Beats Electronics"
        case 0x00CD: return "Microchip Technology Inc."
        case 0x00CE: return "Elgato Systems GmbH"
        case 0x00CF: return "ARCHOS SA"
        case 0x00D0: return "Dexcom, Inc."
        case 0x00D1: return "Polar Electro Europe B.V."
        case 0x00D2: return "Dialog Semiconductor B.V."
        case 0x00D3: return "Taixingbang Technology (HK) Co,. LTD."
        case 0x00D4: return "Kawantech"
        case 0x00D5: return "Austco Communication Systems"
        case 0x00D6: return "Timex Group USA, Inc."
        case 0x00D7: return "Qualcomm Technologies, Inc."
        case 0x00D8: return "Qualcomm Connected Experiences, Inc."
        case 0x00D9: return "Voyetra Turtle Beach"
        case 0x00DA: return "txtr GmbH"
        case 0x00DB: return "Biosentronics"
        case 0x00DC: return "Procter & Gamble"
        case 0x00DD: return "Hosiden Corporation"
        case 0x00DE: return "Muzik LLC"
        case 0x00DF: return "Misfit Wearables Corp"
        case 0x00E0: return "Google"
        case 0x00E1: return "Danlers Ltd"
        case 0x00E2: return "Semilink Inc"
        case 0x00E3: return "inMusic Brands, Inc"
        case 0x00E4: return "L.S. Research Inc."
        case 0x00E5: return "Eden Software Consultants Ltd."
        case 0x00E6: return "Freshtemp"
        case 0x00E7: return "KS Technologies"
        case 0x00E8: return "ACTS Technologies"
        case 0x00E9: return "Vtrack Systems"
        case 0x00EA: return "Nielsen-Kellerman Company"
        case 0x00EB: return "Server Technology Inc."
        case 0x00EC: return "BioResearch Associates"
        case 0x00ED: return "Jolly Logic, LLC"
        case 0x00EE: return "Above Average Outcomes, Inc."
        case 0x00EF: return "Bitsplitters GmbH"
        case 0x00F0: return "PayPal, Inc."
        case 0x00F1: return "Witron Technology Limited"
        case 0x00F2: return "Morse Project Inc."
        case 0x00F3: return "Kent Displays Inc."
        case 0x00F4: return "Nautilus Inc."
        case 0x00F5: return "Smartifier Oy"
        case 0x00F6: return "Elcometer Limited"
        case 0x00F7: return "VSN Technologies, Inc."
        case 0x00F8: return "AceUni Corp., Ltd."
        case 0x00F9: return "StickNFind"
        case 0x00FA: return "Crystal Code AB"
        case 0x00FB: return "KOUKAAM a.s."
        case 0x00FC: return "Delphi Corporation"
        case 0x00FD: return "ValenceTech Limited"
        case 0x00FE: return "Stanley Black and Decker"
        case 0x00FF: return "Typo Products, LLC"
        case 0x0100: return "TomTom International BV"
        case 0x0101: return "Fugoo, Inc."
        case 0x0102: return "Keiser Corporation"
        case 0x0103: return "Bang & Olufsen A/S"
        case 0x0104: return "PLUS Location Systems Pty Ltd"
        case 0x0105: return "Ubiquitous Computing Technology Corporation"
        case 0x0106: return "Innovative Yachtter Solutions"
        case 0x0107: return "William Demant Holding A/S"
        case 0x0108: return "Chicony Electronics Co., Ltd."
        case 0x0109: return "Atus BV"
        case 0x010A: return "Codegate Ltd"
        case 0x010B: return "ERi, Inc"
        case 0x010C: return "Transducers Direct, LLC"
        case 0x010D: return "DENSO TEN LIMITED (formerly Fujitsu Ten LImited)"
        case 0x010E: return "Audi AG"
        case 0x010F: return "HiSilicon Technologies Col, Ltd."
        case 0x0110: return "Nippon Seiki Co., Ltd."
        case 0x0111: return "Steelseries ApS"
        case 0x0112: return "Visybl Inc."
        case 0x0113: return "Openbrain Technologies, Co., Ltd."
        case 0x0114: return "Xensr"
        case 0x0115: return "e.solutions"
        case 0x0116: return "10AK Technologies"
        case 0x0117: return "Wimoto Technologies Inc"
        case 0x0118: return "Radius Networks, Inc."
        case 0x0119: return "Wize Technology Co., Ltd."
        case 0x011A: return "Qualcomm Labs, Inc."
        case 0x011B: return "Hewlett Packard Enterprise"
        case 0x011C: return "Baidu"
        case 0x011D: return "Arendi AG"
        case 0x011E: return "Skoda Auto a.s."
        case 0x011F: return "Volkswagen AG"
        case 0x0120: return "Porsche AG"
        case 0x0121: return "Sino Wealth Electronic Ltd."
        case 0x0122: return "AirTurn, Inc."
        case 0x0123: return "Kinsa, Inc"
        case 0x0124: return "HID Global"
        case 0x0125: return "SEAT es"
        case 0x0126: return "Promethean Ltd."
        case 0x0127: return "Salutica Allied Solutions"
        case 0x0128: return "GPSI Group Pty Ltd"
        case 0x0129: return "Nimble Devices Oy"
        case 0x012A: return "Changzhou Yongse Infotech Co., Ltd."
        case 0x012B: return "SportIQ"
        case 0x012C: return "TEMEC Instruments B.V."
        case 0x012D: return "Sony Corporation"
        case 0x012E: return "ASSA ABLOY"
        case 0x012F: return "Clarion Co. Inc."
        case 0x0130: return "Warehouse Innovations"
        case 0x0131: return "Cypress Semiconductor"
        case 0x0132: return "MADS Inc"
        case 0x0133: return "Blue Maestro Limited"
        case 0x0134: return "Resolution Products, Ltd."
        case 0x0135: return "Aireware LLC"
        case 0x0136: return "Silvair, Inc."
        case 0x0137: return "Prestigio Plaza Ltd."
        case 0x0138: return "NTEO Inc."
        case 0x0139: return "Focus Systems Corporation"
        case 0x013A: return "Tencent Holdings Ltd."
        case 0x013B: return "Allegion"
        case 0x013C: return "Murata Manufacturing Co., Ltd."
        case 0x013D: return "WirelessWERX"
        case 0x013E: return "Nod, Inc."
        case 0x013F: return "B&B Manufacturing Company"
        case 0x0140: return "Alpine Electronics (China) Co., Ltd"
        case 0x0141: return "FedEx Services"
        case 0x0142: return "Grape Systems Inc."
        case 0x0143: return "Bkon Connect"
        case 0x0144: return "Lintech GmbH"
        case 0x0145: return "Novatel Wireless"
        case 0x0146: return "Ciright"
        case 0x0147: return "Mighty Cast, Inc."
        case 0x0148: return "Ambimat Electronics"
        case 0x0149: return "Perytons Ltd."
        case 0x014A: return "Tivoli Audio, LLC"
        case 0x014B: return "Master Lock"
        case 0x014C: return "Mesh-Net Ltd"
        case 0x014D: return "HUIZHOU DESAY SV AUTOMOTIVE CO., LTD."
        case 0x014E: return "Tangerine, Inc."
        case 0x014F: return "B&W Group Ltd."
        case 0x0150: return "Pioneer Corporation"
        case 0x0151: return "OnBeep"
        case 0x0152: return "Vernier Software & Technology"
        case 0x0153: return "ROL Ergo"
        case 0x0154: return "Pebble Technology"
        case 0x0155: return "NETATMO"
        case 0x0156: return "Accumulate AB"
        case 0x0157: return "Anhui Huami Information Technology Co., Ltd."
        case 0x0158: return "Inmite s.r.o."
        case 0x0159: return "ChefSteps, Inc."
        case 0x015A: return "micas AG"
        case 0x015B: return "Biomedical Research Ltd."
        case 0x015C: return "Pitius Tec S.L."
        case 0x015D: return "Estimote, Inc."
        case 0x015E: return "Unikey Technologies, Inc."
        case 0x015F: return "Timer Cap Co."
        case 0x0160: return "AwoX"
        case 0x0161: return "yikes"
        case 0x0162: return "MADSGlobalNZ Ltd."
        case 0x0163: return "PCH International"
        case 0x0164: return "Qingdao Yeelink Information Technology Co., Ltd."
        case 0x0165: return "Milwaukee Tool (Formally Milwaukee Electric Tools)"
        case 0x0166: return "MISHIK Pte Ltd"
        case 0x0167: return "Ascensia Diabetes Care US Inc."
        case 0x0168: return "Spicebox LLC"
        case 0x0169: return "emberlight"
        case 0x016A: return "Cooper-Atkins Corporation"
        case 0x016B: return "Qblinks"
        case 0x016C: return "MYSPHERA"
        case 0x016D: return "LifeScan Inc"
        case 0x016E: return "Volantic AB"
        case 0x016F: return "Podo Labs, Inc"
        case 0x0170: return "Roche Diabetes Care AG"
        case 0x0171: return "Amazon Fulfillment Service"
        case 0x0172: return "Connovate Technology Private Limited"
        case 0x0173: return "Kocomojo, LLC"
        case 0x0174: return "Everykey Inc."
        case 0x0175: return "Dynamic Controls"
        case 0x0176: return "SentriLock"
        case 0x0177: return "I-SYST inc."
        case 0x0178: return "CASIO COMPUTER CO., LTD."
        case 0x0179: return "LAPIS Semiconductor Co., Ltd."
        case 0x017A: return "Telemonitor, Inc."
        case 0x017B: return "taskit GmbH"
        case 0x017C: return "Daimler AG"
        case 0x017D: return "BatAndCat"
        case 0x017E: return "BluDotz Ltd"
        case 0x017F: return "XTel Wireless ApS"
        case 0x0180: return "Gigaset Communications GmbH"
        case 0x0181: return "Gecko Health Innovations, Inc."
        case 0x0182: return "HOP Ubiquitous"
        case 0x0183: return "Walt Disney"
        case 0x0184: return "Nectar"
        case 0x0185: return "bel'apps LLC"
        case 0x0186: return "CORE Lighting Ltd"
        case 0x0187: return "Seraphim Sense Ltd"
        case 0x0188: return "Unico RBC"
        case 0x0189: return "Physical Enterprises Inc."
        case 0x018A: return "Able Trend Technology Limited"
        case 0x018B: return "Konica Minolta, Inc."
        case 0x018C: return "Wilo SE"
        case 0x018D: return "Extron Design Services"
        case 0x018E: return "Fitbit, Inc."
        case 0x018F: return "Fireflies Systems"
        case 0x0190: return "Intelletto Technologies Inc."
        case 0x0191: return "FDK CORPORATION"
        case 0x0192: return "Cloudleaf, Inc"
        case 0x0193: return "Maveric Automation LLC"
        case 0x0194: return "Acoustic Stream Corporation"
        case 0x0195: return "Zuli"
        case 0x0196: return "Paxton Access Ltd"
        case 0x0197: return "WiSilica Inc."
        case 0x0198: return "VENGIT Korlatolt Felelossegu Tarsasag"
        case 0x0199: return "SALTO SYSTEMS S.L."
        case 0x019A: return "TRON Forum (formerly T-Engine Forum)"
        case 0x019B: return "CUBETECH s.r.o."
        case 0x019C: return "Cokiya Incorporated"
        case 0x019D: return "CVS Health"
        case 0x019E: return "Ceruus"
        case 0x019F: return "Strainstall Ltd"
        case 0x01A0: return "Channel Enterprises (HK) Ltd."
        case 0x01A1: return "FIAMM"
        case 0x01A2: return "GIGALANE.CO.,LTD"
        case 0x01A3: return "EROAD"
        case 0x01A4: return "Mine Safety Appliances"
        case 0x01A5: return "Icon Health and Fitness"
        case 0x01A6: return "Wille Engineering (formely as Asandoo GmbH)"
        case 0x01A7: return "ENERGOUS CORPORATION"
        case 0x01A8: return "Taobao"
        case 0x01A9: return "Canon Inc."
        case 0x01AA: return "Geophysical Technology Inc."
        case 0x01AB: return "Facebook, Inc."
        case 0x01AC: return "Trividia Health, Inc."
        case 0x01AD: return "FlightSafety International"
        case 0x01AE: return "Earlens Corporation"
        case 0x01AF: return "Sunrise Micro Devices, Inc."
        case 0x01B0: return "Star Micronics Co., Ltd."
        case 0x01B1: return "Netizens Sp. z o.o."
        case 0x01B2: return "Nymi Inc."
        case 0x01B3: return "Nytec, Inc."
        case 0x01B4: return "Trineo Sp. z o.o."
        case 0x01B5: return "Nest Labs Inc."
        case 0x01B6: return "LM Technologies Ltd"
        case 0x01B7: return "General Electric Company"
        case 0x01B8: return "i+D3 S.L."
        case 0x01B9: return "HANA Micron"
        case 0x01BA: return "Stages Cycling LLC"
        case 0x01BB: return "Cochlear Bone Anchored Solutions AB"
        case 0x01BC: return "SenionLab AB"
        case 0x01BD: return "Syszone Co., Ltd"
        case 0x01BE: return "Pulsate Mobile Ltd."
        case 0x01BF: return "Hong Kong HunterSun Electronic Limited"
        case 0x01C0: return "pironex GmbH"
        case 0x01C1: return "BRADATECH Corp."
        case 0x01C2: return "Transenergooil AG"
        case 0x01C3: return "Bunch"
        case 0x01C4: return "DME Microelectronics"
        case 0x01C5: return "Bitcraze AB"
        case 0x01C6: return "HASWARE Inc."
        case 0x01C7: return "Abiogenix Inc."
        case 0x01C8: return "Poly-Control ApS"
        case 0x01C9: return "Avi-on"
        case 0x01CA: return "Laerdal Medical AS"
        case 0x01CB: return "Fetch My Pet"
        case 0x01CC: return "Sam Labs Ltd."
        case 0x01CD: return "Chengdu Synwing Technology Ltd"
        case 0x01CE: return "HOUWA SYSTEM DESIGN, k.k."
        case 0x01CF: return "BSH"
        case 0x01D0: return "Primus Inter Pares Ltd"
        case 0x01D1: return "August Home, Inc"
        case 0x01D2: return "Gill Electronics"
        case 0x01D3: return "Sky Wave Design"
        case 0x01D4: return "Newlab S.r.l."
        case 0x01D5: return "ELAD srl"
        case 0x01D6: return "G-wearables inc."
        case 0x01D7: return "Squadrone Systems Inc."
        case 0x01D8: return "Code Corporation"
        case 0x01D9: return "Savant Systems LLC"
        case 0x01DA: return "Logitech International SA"
        case 0x01DB: return "Innblue Consulting"
        case 0x01DC: return "iParking Ltd."
        case 0x01DD: return "Koninklijke Philips Electronics N.V."
        case 0x01DE: return "Minelab Electronics Pty Limited"
        case 0x01DF: return "Bison Group Ltd."
        case 0x01E0: return "Widex A/S"
        case 0x01E1: return "Jolla Ltd"
        case 0x01E2: return "Lectronix, Inc."
        case 0x01E3: return "Caterpillar Inc"
        case 0x01E4: return "Freedom Innovations"
        case 0x01E5: return "Dynamic Devices Ltd"
        case 0x01E6: return "Technology Solutions (UK) Ltd"
        case 0x01E7: return "IPS Group Inc."
        case 0x01E8: return "STIR"
        case 0x01E9: return "Sano, Inc."
        case 0x01EA: return "Advanced Application Design, Inc."
        case 0x01EB: return "AutoMap LLC"
        case 0x01EC: return "Spreadtrum Communications Shanghai Ltd"
        case 0x01ED: return "CuteCircuit LTD"
        case 0x01EE: return "Valeo Service"
        case 0x01EF: return "Fullpower Technologies, Inc."
        case 0x01F0: return "KloudNation"
        case 0x01F1: return "Zebra Technologies Corporation"
        case 0x01F2: return "Itron, Inc."
        case 0x01F3: return "The University of Tokyo"
        case 0x01F4: return "UTC Fire and Security"
        case 0x01F5: return "Cool Webthings Limited"
        case 0x01F6: return "DJO Global"
        case 0x01F7: return "Gelliner Limited"
        case 0x01F8: return "Anyka (Guangzhou) Microelectronics Technology Co, LTD"
        case 0x01F9: return "Medtronic Inc."
        case 0x01FA: return "Gozio Inc."
        case 0x01FB: return "Form Lifting, LLC"
        case 0x01FC: return "Wahoo Fitness, LLC"
        case 0x01FD: return "Kontakt Micro-Location Sp. z o.o."
        case 0x01FE: return "Radio Systems Corporation"
        case 0x01FF: return "Freescale Semiconductor, Inc."
        case 0x0200: return "Verifone Systems Pte Ltd. Taiwan Branch"
        case 0x0201: return "AR Timing"
        case 0x0202: return "Rigado LLC"
        case 0x0203: return "Kemppi Oy"
        case 0x0204: return "Tapcentive Inc."
        case 0x0205: return "Smartbotics Inc."
        case 0x0206: return "Otter Products, LLC"
        case 0x0207: return "STEMP Inc."
        case 0x0208: return "LumiGeek LLC"
        case 0x0209: return "InvisionHeart Inc."
        case 0x020A: return "Macnica Inc."
        case 0x020B: return "Jaguar Land Rover Limited"
        case 0x020C: return "CoroWare Technologies, Inc"
        case 0x020D: return "Simplo Technology Co., LTD"
        case 0x020E: return "Omron Healthcare Co., LTD"
        case 0x020F: return "Comodule GMBH"
        case 0x0210: return "ikeGPS"
        case 0x0211: return "Telink Semiconductor Co. Ltd"
        case 0x0212: return "Interplan Co., Ltd"
        case 0x0213: return "Wyler AG"
        case 0x0214: return "IK Multimedia Production srl"
        case 0x0215: return "Lukoton Experience Oy"
        case 0x0216: return "MTI Ltd"
        case 0x0217: return "Tech4home, Lda"
        case 0x0218: return "Hiotech AB"
        case 0x0219: return "DOTT Limited"
        case 0x021A: return "Blue Speck Labs, LLC"
        case 0x021B: return "Cisco Systems, Inc"
        case 0x021C: return "Mobicomm Inc"
        case 0x021D: return "Edamic"
        case 0x021E: return "Goodnet, Ltd"
        case 0x021F: return "Luster Leaf Products Inc"
        case 0x0220: return "Manus Machina BV"
        case 0x0221: return "Mobiquity Networks Inc"
        case 0x0222: return "Praxis Dynamics"
        case 0x0223: return "Philip Morris Products S.A."
        case 0x0224: return "Comarch SA"
        case 0x0226: return "Merlinia A/S"
        case 0x0227: return "LifeBEAM Technologies"
        case 0x0228: return "Twocanoes Labs, LLC"
        case 0x0229: return "Muoverti Limited"
        case 0x022A: return "Stamer Musikanlagen GMBH"
        case 0x022B: return "Tesla Motors"
        case 0x022C: return "Pharynks Corporation"
        case 0x022D: return "Lupine"
        case 0x022E: return "Siemens AG"
        case 0x022F: return "Huami (Shanghai) Culture Communication CO., LTD"
        case 0x0230: return "Foster Electric Company, Ltd"
        case 0x0231: return "ETA SA"
        case 0x0232: return "x-Senso Solutions Kft"
        case 0x0233: return "Shenzhen SuLong Communication Ltd"
        case 0x0234: return "FengFan (BeiJing) Technology Co, Ltd"
        case 0x0235: return "Qrio Inc"
        case 0x0236: return "Pitpatpet Ltd"
        case 0x0237: return "MSHeli s.r.l."
        case 0x0238: return "Trakm8 Ltd"
        case 0x0239: return "JIN CO, Ltd"
        case 0x023A: return "Alatech Tehnology"
        case 0x023B: return "Beijing CarePulse Electronic Technology Co, Ltd"
        case 0x023C: return "Awarepoint"
        case 0x023D: return "ViCentra B.V."
        case 0x023E: return "Raven Industries"
        case 0x023F: return "WaveWare Technologies Inc."
        case 0x0240: return "Argenox Technologies"
        case 0x0241: return "Bragi GmbH"
        case 0x0242: return "16Lab Inc"
        case 0x0243: return "Masimo Corp"
        case 0x0244: return "Iotera Inc"
        case 0x0246: return "ACKme Networks, Inc."
        case 0x0247: return "FiftyThree Inc."
        case 0x0248: return "Parker Hannifin Corp"
        case 0x0249: return "Transcranial Ltd"
        case 0x024A: return "Uwatec AG"
        case 0x024B: return "Orlan LLC"
        case 0x024C: return "Blue Clover Devices"
        case 0x024D: return "M-Way Solutions GmbH"
        case 0x024E: return "Microtronics Engineering GmbH"
        case 0x0250: return "Sapphire Circuits LLC"
        case 0x0251: return "Lumo Bodytech Inc."
        case 0x0252: return "UKC Technosolution"
        case 0x0253: return "Xicato Inc."
        case 0x0254: return "Playbrush"
        case 0x0255: return "Dai Nippon Printing Co., Ltd."
        case 0x0256: return "G24 Power Limited"
        case 0x0257: return "AdBabble Local Commerce Inc."
        case 0x0258: return "Devialet SA"
        case 0x0259: return "ALTYOR"
        case 0x025A: return "University of Applied Sciences Valais/Haute Ecole Valaisanne"
        case 0x025B: return "Five Interactive, LLC dba Zendo"
        case 0x025C: return "NetEase Hangzhou Network co.Ltd."
        case 0x025D: return "Lexmark International Inc."
        case 0x025E: return "Fluke Corporation"
        case 0x025F: return "Yardarm Technologies"
        case 0x0260: return "SensaRx"
        case 0x0261: return "SECVRE GmbH"
        case 0x0262: return "Glacial Ridge Technologies"
        case 0x0263: return "Identiv, Inc."
        case 0x0264: return "DDS, Inc."
        case 0x0265: return "SMK Corporation"
        case 0x0266: return "Schawbel Technologies LLC"
        case 0x0267: return "XMI Systems SA"
        case 0x0268: return "Cerevo"
        case 0x0269: return "Torrox GmbH & Co KG"
        case 0x026A: return "Gemalto"
        case 0x026B: return "DEKA Research & Development Corp."
        case 0x026C: return "Domster Tadeusz Szydlowski"
        case 0x026D: return "Technogym SPA"
        case 0x026E: return "FLEURBAEY BVBA"
        case 0x026F: return "Aptcode Solutions"
        case 0x0270: return "LSI ADL Technology"
        case 0x0271: return "Animas Corp"
        case 0x0272: return "Alps Electric Co., Ltd."
        case 0x0273: return "OCEASOFT"
        case 0x0274: return "Motsai Research"
        case 0x0275: return "Geotab"
        case 0x0276: return "E.G.O. Elektro-Geratebau GmbH"
        case 0x0277: return "bewhere inc"
        case 0x0278: return "Johnson Outdoors Inc"
        case 0x0279: return "steute Schaltgerate GmbH & Co. KG"
        case 0x027A: return "Ekomini inc."
        case 0x027B: return "DEFA AS"
        case 0x027C: return "Aseptika Ltd"
        case 0x027D: return "HUAWEI Technologies Co., Ltd."
        case 0x027E: return "HabitAware, LLC"
        case 0x027F: return "ruwido austria gmbh"
        case 0x0280: return "ITEC corporation"
        case 0x0281: return "StoneL"
        case 0x0282: return "Sonova AG"
        case 0x0283: return "Maven Machines, Inc."
        case 0x0284: return "Synapse Electronics"
        case 0x0285: return "Standard Innovation Inc."
        case 0x0286: return "RF Code, Inc."
        case 0x0287: return "Wally Ventures S.L."
        case 0x0288: return "Willowbank Electronics Ltd"
        case 0x0289: return "SK Telecom"
        case 0x028A: return "Jetro AS"
        case 0x028B: return "Code Gears LTD"
        case 0x028C: return "NANOLINK APS"
        case 0x028D: return "IF, LLC"
        case 0x028E: return "RF Digital Corp"
        case 0x028F: return "Church & Dwight Co., Inc"
        case 0x0290: return "Multibit Oy"
        case 0x0291: return "CliniCloud Inc"
        case 0x0292: return "SwiftSensors"
        case 0x0293: return "Blue Bite"
        case 0x0294: return "ELIAS GmbH"
        case 0x0295: return "Sivantos GmbH"
        case 0x0296: return "Petzl"
        case 0x0297: return "storm power ltd"
        case 0x0298: return "EISST Ltd"
        case 0x0299: return "Inexess Technology Simma KG"
        case 0x029A: return "Currant, Inc."
        case 0x029B: return "C2 Development, Inc."
        case 0x029C: return "Blue Sky Scientific, LLC"
        case 0x029D: return "ALOTTAZS LABS, LLC"
        case 0x029E: return "Kupson spol. s r.o."
        case 0x029F: return "Areus Engineering GmbH"
        case 0x02A0: return "Impossible Camera GmbH"
        case 0x02A1: return "InventureTrack Systems"
        case 0x02A2: return "LockedUp"
        case 0x02A3: return "Itude"
        case 0x02A4: return "Pacific Lock Company"
        case 0x02A5: return "Tendyron Corporation ( ??????????? )"
        case 0x02A6: return "Robert Bosch GmbH"
        case 0x02A7: return "Illuxtron international B.V."
        case 0x02A8: return "miSport Ltd."
        case 0x02A9: return "Chargelib"
        case 0x02AA: return "Doppler Lab"
        case 0x02AB: return "BBPOS Limited"
        case 0x02AC: return "RTB Elektronik GmbH & Co. KG"
        case 0x02AD: return "Rx Networks, Inc."
        case 0x02AE: return "WeatherFlow, Inc."
        case 0x02AF: return "Technicolor USA Inc."
        case 0x02B0: return "Bestechnic(Shanghai),Ltd"
        case 0x02B1: return "Raden Inc"
        case 0x02B2: return "JouZen Oy"
        case 0x02B3: return "CLABER S.P.A."
        case 0x02B4: return "Hyginex, Inc."
        case 0x02B5: return "HANSHIN ELECTRIC RAILWAY CO.,LTD."
        case 0x02B6: return "Schneider Electric"
        case 0x02B7: return "Oort Technologies LLC"
        case 0x02B8: return "Chrono Therapeutics"
        case 0x02B9: return "Rinnai Corporation"
        case 0x02BA: return "Swissprime Technologies AG"
        case 0x02BB: return "Koha.,Co.Ltd"
        case 0x02BC: return "Genevac Ltd"
        case 0x02BD: return "Chemtronics"
        case 0x02BE: return "Seguro Technology Sp. z o.o."
        case 0x02BF: return "Redbird Flight Simulations"
        case 0x02C0: return "Dash Robotics"
        case 0x02C1: return "LINE Corporation"
        case 0x02C2: return "Guillemot Corporation"
        case 0x02C3: return "Techtronic Power Tools Technology Limited"
        case 0x02C4: return "Wilson Sporting Goods"
        case 0x02C5: return "Lenovo (Singapore) Pte Ltd."
        case 0x02C6: return "Ayatan Sensors"
        case 0x02C7: return "Electronics Tomorrow Limited"
        case 0x02C8: return "VASCO Data Security International, Inc."
        case 0x02C9: return "PayRange Inc."
        case 0x02CA: return "ABOV Semiconductor"
        case 0x02CB: return "AINA-Wireless Inc."
        case 0x02CC: return "Eijkelkamp Soil & Water"
        case 0x02CD: return "BMA ergonomics b.v."
        case 0x02CE: return "Teva Branded Pharmaceutical Products R&D, Inc."
        case 0x02CF: return "Anima"
        case 0x02D0: return "3M"
        case 0x02D1: return "Empatica Srl"
        case 0x02D2: return "Afero, Inc."
        case 0x02D3: return "Powercast Corporation"
        case 0x02D4: return "Secuyou ApS"
        case 0x02D5: return "OMRON Corporation"
        case 0x02D6: return "Send Solutions"
        case 0x02D7: return "NIPPON SYSTEMWARE CO.,LTD."
        case 0x02D8: return "Neosfar"
        case 0x02D9: return "Fliegl Agrartechnik GmbH"
        case 0x02DA: return "Gilvader"
        case 0x02DB: return "Digi International Inc (R)"
        case 0x02DC: return "DeWalch Technologies, Inc."
        case 0x02DD: return "Flint Rehabilitation Devices, LLC"
        case 0x02DE: return "Samsung SDS Co., Ltd."
        case 0x02DF: return "Blur Product Development"
        case 0x02E0: return "University of Michigan"
        case 0x02E1: return "Victron Energy BV"
        case 0x02E2: return "NTT docomo"
        case 0x02E3: return "Carmanah Technologies Corp."
        case 0x02E4: return "Bytestorm Ltd."
        case 0x02E5: return "Espressif Incorporated"
        case 0x02E6: return "Unwire"
        case 0x02E7: return "Connected Yard, Inc."
        case 0x02E8: return "American Music Environments"
        case 0x02E9: return "Sensogram Technologies, Inc."
        case 0x02EA: return "Fujitsu Limited"
        case 0x02EB: return "Ardic Technology"
        case 0x02EC: return "Delta Systems, Inc"
        case 0x02EF: return "SMART-INNOVATION.inc"
        case 0x02F1: return "The Idea Cave, LLC"
        case 0x02F2: return "GoPro, Inc."
        case 0x02F3: return "AuthAir, Inc"
        case 0x02F4: return "Vensi, Inc."
        case 0x02F5: return "Indagem Tech LLC"
        case 0x02F6: return "Intemo Technologies"
        case 0x02F7: return "DreamVisions co., Ltd."
        case 0x02F8: return "Runteq Oy Ltd"
        case 0x02FA: return "CoSTAR TEchnologies"
        case 0x02FB: return "Clarius Mobile Health Corp."
        case 0x02FC: return "Shanghai Frequen Microelectronics Co., Ltd."
        case 0x02FD: return "Uwanna, Inc."
        case 0x02FE: return "Lierda Science & Technology Group Co., Ltd."
        case 0x02FF: return "Silicon Laboratories"
        case 0x0300: return "World Moto Inc."
        case 0x0301: return "Giatec Scientific Inc."
        case 0x0302: return "Loop Devices, Inc"
        case 0x0303: return "IACA electronique"
        case 0x0304: return "Proxy Technologies, Inc."
        case 0x0305: return "Swipp ApS"
        case 0x0307: return "FUJI INDUSTRIAL CO.,LTD."
        case 0x0308: return "Surefire, LLC"
        case 0x0309: return "Dolby Labs"
        case 0x030A: return "Ellisys"
        case 0x030B: return "Magnitude Lighting Converters"
        case 0x030C: return "Hilti AG"
        case 0x030D: return "Devdata S.r.l."
        case 0x030E: return "Deviceworx"
        case 0x030F: return "Shortcut Labs"
        case 0x0310: return "SGL Italia S.r.l."
        case 0x0311: return "PEEQ DATA"
        case 0x0314: return "RIIG AI Sp. z o.o."
        case 0x031E: return "Eyefi, Inc."
        case 0x0328: return "Grundfos A/S"
        case 0x032C: return "NIPPON SMT.CO.,Ltd"
        case 0x032E: return "indoormap"
        case 0x0331: return "3flares Technologies Inc."
        case 0x0333: return "Mul-T-Lock"
        case 0x0335: return "Enlighted Inc"
        case 0x0336: return "GISTIC"
        case 0x0337: return "AJP2 Holdings, LLC"
        case 0x033A: return "Appception, Inc."
        case 0x033C: return "Virtuosys"
        case 0x033E: return "Monitra SA"
        case 0x0343: return "Drekker Development Pty. Ltd."
        case 0x0348: return "Arioneo"
        case 0x034D: return "TASER International, Inc."
        case 0x0352: return "iRiding(Xiamen)Technology Co.,Ltd."
        case 0x0358: return "MagniWare Ltd."
        case 0x035D: return "KYS"
        case 0x0362: return "ON Semiconductor"
        case 0x0366: return "BOLTT Sports technologies Private limited"
        case 0x0370: return "Wazombi Labs Oܠ"
        case 0x0371: return "ORBCOMM"
        case 0x0372: return "Nixie Labs, Inc."
        case 0x0373: return "AppNearMe Ltd"
        case 0x0374: return "Holman Industries"
        case 0x0375: return "Expain AS"
        case 0x0376: return "Electronic Temperature Instruments Ltd"
        case 0x0377: return "Plejd AB"
        case 0x0378: return "Propeller Health"
        case 0x0379: return "Shenzhen iMCO Electronic Technology Co.,Ltd"
        case 0x037A: return "Algoria"
        case 0x037B: return "Apption Labs Inc."
        case 0x037C: return "Cronologics Corporation"
        case 0x037D: return "MICRODIA Ltd."
        case 0x037E: return "lulabytes S.L."
        case 0x037F: return "Nestec S.A."
        case 0x0381: return "Sharp Corporation"
        case 0x0382: return "Precision Outcomes Ltd"
        case 0x0383: return "Kronos Incorporated"
        case 0x0384: return "OCOSMOS Co., Ltd."
        case 0x0385: return "Embedded Electronic Solutions Ltd. dba e2Solutions"
        case 0x0386: return "Aterica Inc."
        case 0x0387: return "BluStor PMC, Inc."
        case 0x0388: return "Kapsch TrafficCom AB"
        case 0x0389: return "ActiveBlu Corporation"
        case 0x038A: return "Kohler Mira Limited"
        case 0x038B: return "Noke"
        case 0x038C: return "Appion Inc."
        case 0x038D: return "Resmed Ltd"
        case 0x038E: return "Crownstone B.V."
        case 0x038F: return "Xiaomi Inc."
        case 0x0390: return "INFOTECH s.r.o."
        case 0x0391: return "Thingsquare AB"
        case 0x0392: return "T&D"
        case 0x0393: return "LAVAZZA S.p.A."
        case 0x0394: return "Netclearance Systems, Inc."
        case 0x0395: return "SDATAWAY"
        case 0x0396: return "BLOKS GmbH"
        case 0x0397: return "LEGO System A/S"
        case 0x0398: return "Thetatronics Ltd"
        case 0x0399: return "Nikon Corporation"
        case 0x039A: return "NeST"
        case 0x039B: return "South Silicon Valley Microelectronics"
        case 0x039C: return "ALE International"
        case 0x039D: return "CareView Communications, Inc."
        case 0x039E: return "SchoolBoard Limited"
        case 0x039F: return "Molex Corporation"
        case 0x03A0: return "IVT Wireless Limited"
        case 0x03A1: return "Alpine Labs LLC"
        case 0x03A2: return "Candura Instruments"
        case 0x03A3: return "SmartMovt Technology Co., Ltd"
        case 0x03A4: return "Token Zero Ltd"
        case 0x03A5: return "ACE CAD Enterprise Co., Ltd. (ACECAD)"
        case 0x03A6: return "Medela, Inc"
        case 0x03A7: return "AeroScout"
        case 0x03A8: return "Esrille Inc."
        case 0x03A9: return "THINKERLY SRL"
        case 0x03AA: return "Exon Sp. z o.o."
        case 0x03AB: return "Meizu Technology Co., Ltd."
        case 0x03AC: return "Smablo LTD"
        case 0x03AD: return "XiQ"
        case 0x03AE: return "Allswell Inc."
        case 0x03AF: return "Comm-N-Sense Corp DBA Verigo"
        case 0x03B0: return "VIBRADORM GmbH"
        case 0x03B1: return "Otodata Wireless Network Inc."
        case 0x03B2: return "Propagation Systems Limited"
        case 0x03B3: return "Midwest Instruments & Controls"
        case 0x03B4: return "Alpha Nodus, inc."
        case 0x03B5: return "petPOMM, Inc"
        case 0x03B6: return "Mattel"
        case 0x03B7: return "Airbly Inc."
        case 0x03B8: return "A-Safe Limited"
        case 0x03B9: return "FREDERIQUE CONSTANT SA"
        case 0x03BA: return "Maxscend Microelectronics Company Limited"
        case 0x03BB: return "Abbott Diabetes Care"
        case 0x03BC: return "ASB Bank Ltd"
        case 0x03BD: return "amadas"
        case 0x03BE: return "Applied Science, Inc."
        case 0x03BF: return "iLumi Solutions Inc."
        case 0x03C0: return "Arch Systems Inc."
        case 0x03C1: return "Ember Technologies, Inc."
        case 0x03C2: return "Snapchat Inc"
        case 0x03C3: return "Casambi Technologies Oy"
        case 0x03C4: return "Pico Technology Inc."
        case 0x03C5: return "St. Jude Medical, Inc."
        case 0x03C6: return "Intricon"
        case 0x03C7: return "Structural Health Systems, Inc."
        case 0x03C8: return "Avvel International"
        case 0x03C9: return "Gallagher Group"
        case 0x03CA: return "In2things Automation Pvt. Ltd."
        case 0x03CB: return "SYSDEV Srl"
        case 0x03CC: return "Vonkil Technologies Ltd"
        case 0x03CD: return "Wynd Technologies, Inc."
        case 0x03CE: return "CONTRINEX S.A."
        case 0x03CF: return "MIRA, Inc."
        case 0x03D0: return "Watteam Ltd"
        case 0x03D1: return "Density Inc."
        case 0x03D2: return "IOT Pot India Private Limited"
        case 0x03D3: return "Sigma Connectivity AB"
        case 0x03D4: return "PEG PEREGO SPA"
        case 0x03D5: return "Wyzelink Systems Inc."
        case 0x03D6: return "Yota Devices LTD"
        case 0x03D7: return "FINSECUR"
        case 0x03D8: return "Zen-Me Labs Ltd"
        case 0x03D9: return "3IWare Co., Ltd."
        case 0x03DA: return "EnOcean GmbH"
        case 0x03DB: return "Instabeat, Inc"
        case 0x03DC: return "Nima Labs"
        case 0x03DD: return "Andreas Stihl AG & Co. KG"
        case 0x03DE: return "Nathan Rhoades LLC"
        case 0x03DF: return "Grob Technologies, LLC"
        case 0x03E0: return "Actions (Zhuhai) Technology Co., Limited"
        case 0x03E1: return "SPD Development Company Ltd"
        case 0x03E2: return "Sensoan Oy"
        case 0x03E3: return "Qualcomm Life Inc"
        case 0x03E4: return "Chip-ing AG"
        case 0x03E5: return "ffly4u"
        case 0x03E6: return "IoT Instruments Oy"
        case 0x03E7: return "TRUE Fitness Technology"
        case 0x03E8: return "Reiner Kartengeraete GmbH & Co. KG."
        case 0x03E9: return "SHENZHEN LEMONJOY TECHNOLOGY CO., LTD."
        case 0x03EA: return "Hello Inc."
        case 0x03EB: return "Evollve Inc."
        case 0x03EC: return "Jigowatts Inc."
        case 0x03ED: return "BASIC MICRO.COM,INC."
        case 0x03EE: return "CUBE TECHNOLOGIES"
        case 0x03EF: return "foolography GmbH"
        case 0x03F0: return "CLINK"
        case 0x03F1: return "Hestan Smart Cooking Inc."
        case 0x03F2: return "WindowMaster A/S"
        case 0x03F3: return "Flowscape AB"
        case 0x03F4: return "PAL Technologies Ltd"
        case 0x03F5: return "WHERE, Inc."
        case 0x03F6: return "Iton Technology Corp."
        case 0x03F7: return "Owl Labs Inc."
        case 0x03F8: return "Rockford Corp."
        case 0x03F9: return "Becon Technologies Co.,Ltd."
        case 0x03FA: return "Vyassoft Technologies Inc"
        case 0x03FB: return "Nox Medical"
        case 0x03FC: return "Kimberly-Clark"
        case 0x03FD: return "Trimble Navigation Ltd."
        case 0x03FE: return "Littelfuse"
        case 0x03FF: return "Withings"
        case 0x0400: return "i-developer IT Beratung UG"
        case 0x0402: return "Sears Holdings Corporation"
        case 0x0403: return "Gantner Electronic GmbH"
        case 0x0404: return "Authomate Inc"
        case 0x0405: return "Vertex International, Inc."
        case 0x0406: return "Airtago"
        case 0x0407: return "Swiss Audio SA"
        case 0x0408: return "ToGetHome Inc."
        case 0x0409: return "AXIS"
        case 0x040A: return "Openmatics"
        case 0x040B: return "Jana Care Inc."
        case 0x040C: return "Senix Corporation"
        case 0x040D: return "NorthStar Battery Company, LLC"
        case 0x040E: return "SKF (U.K.) Limited"
        case 0x040F: return "CO-AX Technology, Inc."
        case 0x0410: return "Fender Musical Instruments"
        case 0x0411: return "Luidia Inc"
        case 0x0412: return "SEFAM"
        case 0x0413: return "Wireless Cables Inc"
        case 0x0414: return "Lightning Protection International Pty Ltd"
        case 0x0415: return "Uber Technologies Inc"
        case 0x0416: return "SODA GmbH"
        case 0x0417: return "Fatigue Science"
        case 0x0418: return "Alpine Electronics Inc."
        case 0x0419: return "Novalogy LTD"
        case 0x041A: return "Friday Labs Limited"
        case 0x041B: return "OrthoAccel Technologies"
        case 0x041C: return "WaterGuru, Inc."
        case 0x041D: return "Benning Elektrotechnik und Elektronik GmbH & Co. KG"
        case 0x041E: return "Dell Computer Corporation"
        case 0x041F: return "Kopin Corporation"
        case 0x0420: return "TecBakery GmbH"
        case 0x0421: return "Backbone Labs, Inc."
        case 0x0422: return "DELSEY SA"
        case 0x0423: return "Chargifi Limited"
        case 0x0424: return "Trainesense Ltd."
        case 0x0425: return "Unify Software and Solutions GmbH & Co. KG"
        case 0x0426: return "Husqvarna AB"
        case 0x0427: return "Focus fleet and fuel management inc"
        case 0x0428: return "SmallLoop, LLC"
        case 0x0429: return "Prolon Inc."
        case 0x042A: return "BD Medical"
        case 0x042B: return "iMicroMed Incorporated"
        case 0x042C: return "Ticto N.V."
        case 0x042D: return "Meshtech AS"
        case 0x042E: return "MemCachier Inc."
        case 0x042F: return "Danfoss A/S"
        case 0x0430: return "SnapStyk Inc."
        case 0x0431: return "Amway Corporation"
        case 0x0432: return "Silk Labs, Inc."
        case 0x0433: return "Pillsy Inc."
        case 0x0434: return "Hatch Baby, Inc."
        case 0x0435: return "Blocks Wearables Ltd."
        case 0x0436: return "Drayson Technologies (Europe) Limited"
        case 0x0437: return "eBest IOT Inc."
        case 0x0438: return "Helvar Ltd"
        case 0x0439: return "Radiance Technologies"
        case 0x043A: return "Nuheara Limited"
        case 0x043B: return "Appside co., ltd."
        case 0x043C: return "DeLaval"
        case 0x043D: return "Coiler Corporation"
        case 0x043E: return "Thermomedics, Inc."
        case 0x043F: return "Tentacle Sync GmbH"
        case 0x0440: return "Valencell, Inc."
        case 0x0441: return "iProtoXi Oy"
        case 0x0442: return "SECOM CO., LTD."
        case 0x0443: return "Tucker International LLC"
        case 0x0444: return "Metanate Limited"
        case 0x0445: return "Kobian Canada Inc."
        case 0x0446: return "NETGEAR, Inc."
        case 0x0447: return "Fabtronics Australia Pty Ltd"
        case 0x0448: return "Grand Centrix GmbH"
        case 0x0449: return "1UP USA.com llc"
        case 0x044A: return "SHIMANO INC."
        case 0x044B: return "Nain Inc."
        case 0x044C: return "LifeStyle Lock, LLC"
        case 0x044D: return "VEGA Grieshaber KG"
        case 0x044E: return "Xtrava Inc."
        case 0x044F: return "TTS Tooltechnic Systems AG & Co. KG"
        case 0x0450: return "Teenage Engineering AB"
        case 0x0451: return "Tunstall Nordic AB"
        case 0x0452: return "Svep Design Center AB"
        case 0x0453: return "GreenPeak Technologies BV"
        case 0x0454: return "Sphinx Electronics GmbH & Co KG"
        case 0x0455: return "Atomation"
        case 0x0456: return "Nemik Consulting Inc"
        case 0x0457: return "RF INNOVATION"
        case 0x0458: return "Mini Solution Co., Ltd."
        case 0x0459: return "Lumenetix, Inc"
        case 0x045A: return "2048450 Ontario Inc"
        case 0x045B: return "SPACEEK LTD"
        case 0x045C: return "Delta T Corporation"
        case 0x045D: return "Boston Scientific Corporation"
        case 0x045E: return "Nuviz, Inc."
        case 0x045F: return "Real Time Automation, Inc."
        case 0x0460: return "Kolibree"
        case 0x0461: return "vhf elektronik GmbH"
        case 0x0462: return "Bonsai Systems GmbH"
        case 0x0463: return "Fathom Systems Inc."
        case 0x0464: return "Bellman & Symfon"
        case 0x0465: return "International Forte Group LLC"
        case 0x0466: return "CycleLabs Solutions inc."
        case 0x0467: return "Codenex Oy"
        case 0x0468: return "Kynesim Ltd"
        case 0x0469: return "Palago AB"
        case 0x046A: return "INSIGMA INC."
        case 0x046B: return "PMD Solutions"
        case 0x046C: return "Qingdao Realtime Technology Co., Ltd."
        case 0x046D: return "BEGA Gantenbrink-Leuchten KG"
        case 0x046E: return "Pambor Ltd."
        case 0x046F: return "Develco Products A/S"
        case 0x0470: return "iDesign s.r.l."
        case 0x0471: return "TiVo Corp"
        case 0x0472: return "Control-J Pty Ltd"
        case 0x0473: return "Steelcase, Inc."
        case 0x0474: return "iApartment co., ltd."
        case 0x0475: return "Icom inc."
        case 0x0476: return "Oxstren Wearable Technologies Private Limited"
        case 0x0477: return "Blue Spark Technologies"
        case 0x0478: return "FarSite Communications Limited"
        case 0x0479: return "mywerk system GmbH"
        case 0x047A: return "Sinosun Technology Co., Ltd."
        case 0x047B: return "MIYOSHI ELECTRONICS CORPORATION"
        case 0x047C: return "POWERMAT LTD"
        case 0x047D: return "Occly LLC"
        case 0x047E: return "OurHub Dev IvS"
        case 0x047F: return "Pro-Mark, Inc."
        case 0x0480: return "Dynometrics Inc."
        case 0x0481: return "Quintrax Limited"
        case 0x0482: return "POS Tuning Udo Vosshenrich GmbH & Co. KG"
        case 0x0483: return "Multi Care Systems B.V."
        case 0x0484: return "Revol Technologies Inc"
        case 0x0485: return "SKIDATA AG"
        case 0x0486: return "DEV TECNOLOGIA INDUSTRIA, COMERCIO E MANUTENCAO DE EQUIPAMENTOS LTDA. - ME"
        case 0x0487: return "Centrica Connected Home"
        case 0x0488: return "Automotive Data Solutions Inc"
        case 0x0489: return "Igarashi Engineering"
        case 0x048A: return "Taelek Oy"
        case 0x048B: return "CP Electronics Limited"
        case 0x048C: return "Vectronix AG"
        case 0x048D: return "S-Labs Sp. z o.o."
        case 0x048E: return "Companion Medical, Inc."
        case 0x048F: return "BlueKitchen GmbH"
        case 0x0490: return "Matting AB"
        case 0x0491: return "SOREX - Wireless Solutions GmbH"
        case 0x0492: return "ADC Technology, Inc."
        case 0x0493: return "Lynxemi Pte Ltd"
        case 0x0494: return "SENNHEISER electronic GmbH & Co. KG"
        case 0x0495: return "LMT Mercer Group, Inc"
        case 0x0496: return "Polymorphic Labs LLC"
        case 0x0497: return "Cochlear Limited"
        case 0x0498: return "METER Group, Inc. USA"
        case 0x0499: return "Ruuvi Innovations Ltd."
        case 0x049A: return "Situne AS"
        case 0x049B: return "nVisti, LLC"
        case 0x049C: return "DyOcean"
        case 0x049D: return "Uhlmann & Zacher GmbH"
        case 0x049E: return "AND!XOR LLC"
        case 0x049F: return "tictote AB"
        case 0x04A0: return "Vypin, LLC"
        case 0x04A1: return "PNI Sensor Corporation"
        case 0x04A2: return "ovrEngineered, LLC"
        case 0x04A3: return "GT-tronics HK Ltd"
        case 0x04A4: return "Herbert Waldmann GmbH & Co. KG"
        case 0x04A5: return "Guangzhou FiiO Electronics Technology Co.,Ltd"
        case 0x04A6: return "Vinetech Co., Ltd"
        case 0x04A7: return "Dallas Logic Corporation"
        case 0x04A8: return "BioTex, Inc."
        case 0x04A9: return "DISCOVERY SOUND TECHNOLOGY, LLC"
        case 0x04AA: return "LINKIO SAS"
        case 0x04AB: return "Harbortronics, Inc."
        case 0x04AC: return "Undagrid B.V."
        case 0x04AD: return "Shure Inc"
        case 0x04AE: return "ERM Electronic Systems LTD"
        case 0x04AF: return "BIOROWER Handelsagentur GmbH"
        case 0x04B0: return "Weba Sport und Med. Artikel GmbH"
        case 0x04B1: return "Kartographers Technologies Pvt. Ltd."
        case 0x04B2: return "The Shadow on the Moon"
        case 0x04B3: return "mobike (Hong Kong) Limited"
        case 0x04B4: return "Inuheat Group AB"
        case 0x04B5: return "Swiftronix AB"
        case 0x04B6: return "Diagnoptics Technologies"
        case 0x04B7: return "Analog Devices, Inc."
        case 0x04B8: return "Soraa Inc."
        case 0x04B9: return "CSR Building Products Limited"
        case 0x04BA: return "Crestron Electronics, Inc."
        case 0x04BB: return "Neatebox Ltd"
        case 0x04BC: return "Draegerwerk AG & Co. KGaA"
        case 0x04BD: return "AlbynMedical"
        case 0x04BE: return "Averos FZCO"
        case 0x04BF: return "VIT Initiative, LLC"
        case 0x04C0: return "Statsports International"
        case 0x04C1: return "Sospitas, s.r.o."
        case 0x04C2: return "Dmet Products Corp."
        case 0x04C3: return "Mantracourt Electronics Limited"
        case 0x04C4: return "TeAM Hutchins AB"
        case 0x04C5: return "Seibert Williams Glass, LLC"
        case 0x04C6: return "Insta GmbH"
        case 0x04C7: return "Svantek Sp. z o.o."
        case 0x04C8: return "Shanghai Flyco Electrical Appliance Co., Ltd."
        case 0x04C9: return "Thornwave Labs Inc"
        case 0x04CA: return "Steiner-Optik GmbH"
        case 0x04CB: return "Novo Nordisk A/S"
        case 0x04CC: return "Enflux Inc."
        case 0x04CD: return "Safetech Products LLC"
        case 0x04CE: return "GOOOLED S.R.L."
        case 0x04CF: return "DOM Sicherheitstechnik GmbH & Co. KG"
        case 0x04D0: return "Olympus Corporation"
        case 0x04D1: return "KTS GmbH"
        case 0x04D2: return "Anloq Technologies Inc."
        case 0x04D3: return "Queercon, Inc"
        case 0x04D4: return "5th Element Ltd"
        case 0x04D5: return "Gooee Limited"
        case 0x04D6: return "LUGLOC LLC"
        case 0x04D7: return "Blincam, Inc."
        case 0x04D8: return "FUJIFILM Corporation"
        case 0x04D9: return "RandMcNally"
        case 0x04DA: return "Franceschi Marina snc"
        case 0x04DB: return "Engineered Audio, LLC."
        case 0x04DC: return "IOTTIVE (OPC) PRIVATE LIMITED"
        case 0x04DD: return "4MOD Technology"
        case 0x04DE: return "Lutron Electronics Co., Inc."
        case 0x04DF: return "Emerson"
        case 0x04E0: return "Guardtec, Inc."
        case 0x04E1: return "REACTEC LIMITED"
        case 0x04E2: return "EllieGrid"
        case 0x04E3: return "Under Armour"
        case 0x04E4: return "Woodenshark"
        case 0x04E5: return "Avack Oy"
        case 0x04E6: return "Smart Solution Technology, Inc."
        case 0x04E7: return "REHABTRONICS INC."
        case 0x04E8: return "STABILO International"
        case 0x04E9: return "Busch Jaeger Elektro GmbH"
        case 0x04EA: return "Pacific Bioscience Laboratories, Inc"
        case 0x04EB: return "Bird Home Automation GmbH"
        case 0x04EC: return "Motorola Solutions"
        case 0x04ED: return "R9 Technology, Inc."
        case 0x04EE: return "Auxivia"
        case 0x04EF: return "DaisyWorks, Inc"
        case 0x04F0: return "Kosi Limited"
        case 0x04F1: return "Theben AG"
        case 0x04F2: return "InDreamer Techsol Private Limited"
        case 0x04F3: return "Cerevast Medical"
        case 0x04F4: return "ZanCompute Inc."
        case 0x04F5: return "Pirelli Tyre S.P.A."
        case 0x04F6: return "McLear Limited"
        case 0x04F7: return "Shenzhen Huiding Technology Co.,Ltd."
        case 0x04F8: return "Convergence Systems Limited"
        case 0x04F9: return "Interactio"
        case 0x04FA: return "Androtec GmbH"
        case 0x04FB: return "Benchmark Drives GmbH & Co. KG"
        case 0x04FC: return "SwingLync L. L. C."
        case 0x04FD: return "Tapkey GmbH"
        case 0x04FE: return "Woosim Systems Inc."
        case 0x04FF: return "Microsemi Corporation"
        case 0x0500: return "Wiliot LTD."
        case 0x0501: return "Polaris IND"
        case 0x0502: return "Specifi-Kali LLC"
        case 0x0503: return "Locoroll, Inc"
        case 0x0504: return "PHYPLUS Inc"
        case 0x0505: return "Inplay Technologies LLC"
        case 0x0506: return "Hager"
        case 0x0507: return "Yellowcog"
        case 0x0508: return "Axes System sp. z o. o."
        case 0x0509: return "myLIFTER Inc."
        case 0x050A: return "Shake-on B.V."
        case 0x050B: return "Vibrissa Inc."
        case 0x050C: return "OSRAM GmbH"
        case 0x050D: return "TRSystems GmbH"
        case 0x050E: return "Yichip Microelectronics (Hangzhou) Co.,Ltd."
        case 0x050F: return "Foundation Engineering LLC"
        case 0x0510: return "UNI-ELECTRONICS, INC."
        case 0x0511: return "Brookfield Equinox LLC"
        case 0x0512: return "Soprod SA"
        case 0x0513: return "9974091 Canada Inc."
        case 0x0514: return "FIBRO GmbH"
        case 0x0515: return "RB Controls Co., Ltd."
        case 0x0516: return "Footmarks"
        case 0x0517: return "Amtronic Sverige AB (formerly Amcore AB)"
        case 0x0518: return "MAMORIO.inc"
        case 0x0519: return "Tyto Life LLC"
        case 0x051A: return "Leica Camera AG"
        case 0x051B: return "Angee Technologies Ltd."
        case 0x051C: return "EDPS"
        case 0x051D: return "OFF Line Co., Ltd."
        case 0x051E: return "Detect Blue Limited"
        case 0x051F: return "Setec Pty Ltd"
        case 0x0520: return "Target Corporation"
        case 0x0521: return "IAI Corporation"
        case 0x0522: return "NS Tech, Inc."
        case 0x0523: return "MTG Co., Ltd."
        case 0x0524: return "Hangzhou iMagic Technology Co., Ltd"
        case 0x0525: return "HONGKONG NANO IC TECHNOLOGIES CO., LIMITED"
        case 0x0526: return "Honeywell International Inc."
        case 0x0527: return "Albrecht JUNG"
        case 0x0528: return "Lunera Lighting Inc."
        case 0x0529: return "Lumen UAB"
        case 0x052A: return "Keynes Controls Ltd"
        case 0x052B: return "Novartis AG"
        case 0x052C: return "Geosatis SA"
        case 0x052D: return "EXFO, Inc."
        case 0x052E: return "LEDVANCE GmbH"
        case 0x052F: return "Center ID Corp."
        case 0x0530: return "Adolene, Inc."
        case 0x0531: return "D&M Holdings Inc."
        case 0x0532: return "CRESCO Wireless, Inc."
        case 0x0533: return "Nura Operations Pty Ltd"
        case 0x0534: return "Frontiergadget, Inc."
        case 0x0535: return "Smart Component Technologies Limited"
        case 0x0536: return "ZTR Control Systems LLC"
        case 0x0537: return "MetaLogics Corporation"
        case 0x0538: return "Medela AG"
        case 0x0539: return "OPPLE Lighting Co., Ltd"
        case 0x053A: return "Savitech Corp.,"
        case 0x053B: return "prodigy"
        case 0x053C: return "Screenovate Technologies Ltd"
        case 0x053D: return "TESA SA"
        case 0x053E: return "CLIM8 LIMITED"
        case 0x053F: return "Silergy Corp"
        case 0x0540: return "SilverPlus, Inc"
        case 0x0541: return "Sharknet srl"
        case 0x0542: return "Mist Systems, Inc."
        case 0x0543: return "MIWA LOCK CO.,Ltd"
        case 0x0544: return "OrthoSensor, Inc."
        case 0x0545: return "Candy Hoover Group s.r.l"
        case 0x0546: return "Apexar Technologies S.A."
        case 0x0547: return "LOGICDATA d.o.o."
        case 0x0548: return "Knick Elektronische Messgeraete GmbH & Co. KG"
        case 0x0549: return "Smart Technologies and Investment Limited"
        case 0x054A: return "Linough Inc."
        case 0x054B: return "Advanced Electronic Designs, Inc."
        case 0x054C: return "Carefree Scott Fetzer Co Inc"
        case 0x054D: return "Sensome"
        case 0x054E: return "FORTRONIK storitve d.o.o."
        case 0x054F: return "Sinnoz"
        case 0x0550: return "Versa Networks, Inc."
        case 0x0551: return "Sylero"
        case 0x0552: return "Avempace SARL"
        case 0x0553: return "Nintendo Co., Ltd."
        case 0x0554: return "National Instruments"
        case 0x0555: return "KROHNE Messtechnik GmbH"
        case 0x0556: return "Otodynamics Ltd"
        case 0x0557: return "Arwin Technology Limited"
        case 0x0558: return "benegear, inc."
        case 0x0559: return "Newcon Optik"
        case 0x055A: return "CANDY HOUSE, Inc."
        case 0x055B: return "FRANKLIN TECHNOLOGY INC"
        case 0x055C: return "Lely"
        case 0x055D: return "Valve Corporation"
        case 0x055E: return "Hekatron Vertriebs GmbH"
        case 0x055F: return "PROTECH S.A.S. DI GIRARDI ANDREA & C."
        case 0x0560: return "Sarita CareTech APS (formerly Sarita CareTech IVS)"
        case 0x0561: return "Finder S.p.A."
        case 0x0562: return "Thalmic Labs Inc."
        case 0x0563: return "Steinel Vertrieb GmbH"
        case 0x0564: return "Beghelli Spa"
        case 0x0565: return "Beijing Smartspace Technologies Inc."
        case 0x0566: return "CORE TRANSPORT TECHNOLOGIES NZ LIMITED"
        case 0x0567: return "Xiamen Everesports Goods Co., Ltd"
        case 0x0568: return "Bodyport Inc."
        case 0x0569: return "Audionics System, INC."
        case 0x056A: return "Flipnavi Co.,Ltd."
        case 0x056B: return "Rion Co., Ltd."
        case 0x056C: return "Long Range Systems, LLC"
        case 0x056D: return "Redmond Industrial Group LLC"
        case 0x056E: return "VIZPIN INC."
        case 0x056F: return "BikeFinder AS"
        case 0x0570: return "Consumer Sleep Solutions LLC"
        case 0x0571: return "PSIKICK, INC."
        case 0x0572: return "AntTail.com"
        case 0x0573: return "Lighting Science Group Corp."
        case 0x0574: return "AFFORDABLE ELECTRONICS INC"
        case 0x0575: return "Integral Memroy Plc"
        case 0x0576: return "Globalstar, Inc."
        case 0x0577: return "True Wearables, Inc."
        case 0x0578: return "Wellington Drive Technologies Ltd"
        case 0x0579: return "Ensemble Tech Private Limited"
        case 0x057A: return "OMNI Remotes"
        case 0x057B: return "Duracell U.S. Operations Inc."
        case 0x057C: return "Toor Technologies LLC"
        case 0x057D: return "Instinct Performance"
        case 0x057E: return "Beco, Inc"
        case 0x057F: return "Scuf Gaming International, LLC"
        case 0x0580: return "ARANZ Medical Limited"
        case 0x0581: return "LYS TECHNOLOGIES LTD"
        case 0x0582: return "Breakwall Analytics, LLC"
        case 0x0583: return "Code Blue Communications"
        case 0x0584: return "Gira Giersiepen GmbH & Co. KG"
        case 0x0585: return "Hearing Lab Technology"
        case 0x0586: return "LEGRAND"
        case 0x0587: return "Derichs GmbH"
        case 0x0588: return "ALT-TEKNIK LLC"
        case 0x0589: return "Star Technologies"
        case 0x058A: return "START TODAY CO.,LTD."
        case 0x058B: return "Maxim Integrated Products"
        case 0x058C: return "MERCK Kommanditgesellschaft auf Aktien"
        case 0x058D: return "Jungheinrich Aktiengesellschaft"
        case 0x058E: return "Oculus VR, LLC"
        case 0x058F: return "HENDON SEMICONDUCTORS PTY LTD"
        case 0x0590: return "Pur3 Ltd"
        case 0x0591: return "Viasat Group S.p.A."
        case 0x0592: return "IZITHERM"
        case 0x0593: return "Spaulding Clinical Research"
        case 0x0594: return "Kohler Company"
        case 0x0595: return "Inor Process AB"
        case 0x0596: return "My Smart Blinds"
        case 0x0597: return "RadioPulse Inc"
        case 0x0598: return "rapitag GmbH"
        case 0x0599: return "Lazlo326, LLC."
        case 0x059A: return "Teledyne Lecroy, Inc."
        case 0x059B: return "Dataflow Systems Limited"
        case 0x059C: return "Macrogiga Electronics"
        case 0x059D: return "Tandem Diabetes Care"
        case 0x059E: return "Polycom, Inc."
        case 0x059F: return "Fisher & Paykel Healthcare"
        case 0x05A0: return "RCP Software Oy"
        case 0x05A1: return "Shanghai Xiaoyi Technology Co.,Ltd."
        case 0x05A2: return "ADHERIUM(NZ) LIMITED"
        case 0x05A3: return "Axiomware Systems Incorporated"
        case 0x05A4: return "O. E. M. Controls, Inc."
        case 0x05A5: return "Kiiroo BV"
        case 0x05A6: return "Telecon Mobile Limited"
        case 0x05A7: return "Sonos Inc"
        case 0x05A8: return "Tom Allebrandi Consulting"
        case 0x05A9: return "Monidor"
        case 0x05AA: return "Tramex Limited"
        case 0x05AB: return "Nofence AS"
        case 0x05AC: return "GoerTek Dynaudio Co., Ltd."
        case 0x05AD: return "INIA"
        case 0x05AE: return "CARMATE MFG.CO.,LTD"
        case 0x05AF: return "ONvocal"
        case 0x05B0: return "NewTec GmbH"
        case 0x05B1: return "Medallion Instrumentation Systems"
        case 0x05B2: return "CAREL INDUSTRIES S.P.A."
        case 0x05B3: return "Parabit Systems, Inc."
        case 0x05B4: return "White Horse Scientific ltd"
        case 0x05B5: return "verisilicon"
        case 0x05B6: return "Elecs Industry Co.,Ltd."
        case 0x05B7: return "Beijing Pinecone Electronics Co.,Ltd."
        case 0x05B8: return "Ambystoma Labs Inc."
        case 0x05B9: return "Suzhou Pairlink Network Technology"
        case 0x05BA: return "igloohome"
        case 0x05BB: return "Oxford Metrics plc"
        case 0x05BC: return "Leviton Mfg. Co., Inc."
        case 0x05BD: return "ULC Robotics Inc."
        case 0x05BE: return "RFID Global by Softwork SrL"
        case 0x05BF: return "Real-World-Systems Corporation"
        case 0x05C0: return "Nalu Medical, Inc."
        case 0x05C1: return "P.I.Engineering"
        case 0x05C2: return "Grote Industries"
        case 0x05C3: return "Runtime, Inc."
        case 0x05C4: return "Codecoup sp. z o.o. sp. k."
        case 0x05C5: return "SELVE GmbH & Co. KG"
        case 0x05C6: return "Smart Animal Training Systems, LLC"
        case 0x05C7: return "Lippert Components, INC"
        case 0x05C8: return "SOMFY SAS"
        case 0x05C9: return "TBS Electronics B.V."
        case 0x05CA: return "MHL Custom Inc"
        case 0x05CB: return "LucentWear LLC"
        case 0x05CC: return "WATTS ELECTRONICS"
        case 0x05CD: return "RJ Brands LLC"
        case 0x05CE: return "V-ZUG Ltd"
        case 0x05CF: return "Biowatch SA"
        case 0x05D0: return "Anova Applied Electronics"
        case 0x05D1: return "Lindab AB"
        case 0x05D2: return "frogblue TECHNOLOGY GmbH"
        case 0x05D3: return "Acurable Limited"
        case 0x05D4: return "LAMPLIGHT Co., Ltd."
        case 0x05D5: return "TEGAM, Inc."
        case 0x05D6: return "Zhuhai Jieli technology Co.,Ltd"
        case 0x05D7: return "modum.io AG"
        case 0x05D8: return "Farm Jenny LLC"
        case 0x05D9: return "Toyo Electronics Corporation"
        case 0x05DA: return "Applied Neural Research Corp"
        case 0x05DB: return "Avid Identification Systems, Inc."
        case 0x05DC: return "Petronics Inc."
        case 0x05DD: return "essentim GmbH"
        case 0x05DE: return "QT Medical INC."
        case 0x05DF: return "VIRTUALCLINIC.DIRECT LIMITED"
        case 0x05E0: return "Viper Design LLC"
        case 0x05E1: return "Human, Incorporated"
        case 0x05E2: return "stAPPtronics GmbH"
        case 0x05E3: return "Elemental Machines, Inc."
        case 0x05E4: return "Taiyo Yuden Co., Ltd"
        case 0x05E5: return "INEO ENERGY& SYSTEMS"
        case 0x05E6: return "Motion Instruments Inc."
        case 0x05E7: return "PressurePro"
        case 0x05E8: return "COWBOY"
        case 0x05E9: return "iconmobile GmbH"
        case 0x05EA: return "ACS-Control-System GmbH"
        case 0x05EB: return "Bayerische Motoren Werke AG"
        case 0x05EC: return "Gycom Svenska AB"
        case 0x05ED: return "Fuji Xerox Co., Ltd"
        case 0x05EE: return "Glide Inc."
        case 0x05EF: return "SIKOM AS"
        case 0x05F0: return "beken"
        case 0x05F1: return "The Linux Foundation"
        case 0x05F2: return "Try and E CO.,LTD."
        case 0x05F3: return "SeeScan"
        case 0x05F4: return "Clearity, LLC"
        case 0x05F5: return "GS TAG"
        case 0x05F6: return "DPTechnics"
        case 0x05F7: return "TRACMO, INC."
        case 0x05F8: return "Anki Inc."
        case 0x05F9: return "Hagleitner Hygiene International GmbH"
        case 0x05FA: return "Konami Sports Life Co., Ltd."
        case 0x05FB: return "Arblet Inc."
        case 0x05FC: return "Masbando GmbH"
        case 0x05FD: return "Innoseis"
        case 0x05FE: return "Niko"
        case 0x05FF: return "Wellnomics Ltd"
        case 0x0600: return "iRobot Corporation"
        case 0x0601: return "Schrader Electronics"
        case 0x0602: return "Geberit International AG"
        case 0x0603: return "Fourth Evolution Inc"
        case 0x0604: return "Cell2Jack LLC"
        case 0x0605: return "FMW electronic Futterer u. Maier-Wolf OHG"
        case 0x0606: return "John Deere"
        case 0x0607: return "Rookery Technology Ltd"
        case 0x0608: return "KeySafe-Cloud"
        case 0x0609: return "BUCHI Labortechnik AG"
        case 0x060A: return "IQAir AG"
        case 0x060B: return "Triax Technologies Inc"
        case 0x060C: return "Vuzix Corporation"
        case 0x060D: return "TDK Corporation"
        case 0x060E: return "Blueair AB"
        case 0x060F: return "Signify Netherlands"
        case 0x0610: return "ADH GUARDIAN USA LLC"
        case 0x0611: return "Beurer GmbH"
        case 0x0612: return "Playfinity AS"
        case 0x0613: return "Hans Dinslage GmbH"
        case 0x0614: return "OnAsset Intelligence, Inc."
        case 0x0615: return "INTER ACTION Corporation"
        case 0x0616: return "OS42 UG (haftungsbeschraenkt)"
        case 0x0617: return "WIZCONNECTED COMPANY LIMITED"
        case 0x0618: return "Audio-Technica Corporation"
        case 0x0619: return "Six Guys Labs, s.r.o."
        case 0x061A: return "R.W. Beckett Corporation"
        case 0x061B: return "silex technology, inc."
        case 0x061C: return "Univations Limited"
        case 0x061D: return "SENS Innovation ApS"
        case 0x061E: return "Diamond Kinetics, Inc."
        case 0x061F: return "Phrame Inc."
        case 0x0620: return "Forciot Oy"
        case 0x0621: return "Noordung d.o.o."
        case 0x0622: return "Beam Labs, LLC"
        case 0x0623: return "Philadelphia Scientific (U.K.) Limited"
        case 0x0624: return "Biovotion AG"
        case 0x0625: return "Square Panda, Inc."
        case 0x0626: return "Amplifico"
        case 0x0627: return "WEG S.A."
        case 0x0628: return "Ensto Oy"
        case 0x0629: return "PHONEPE PVT LTD"
        case 0x062A: return "Lunatico Astronomia SL"
        case 0x062B: return "MinebeaMitsumi Inc."
        case 0x062C: return "ASPion GmbH"
        case 0x062D: return "Vossloh-Schwabe Deutschland GmbH"
        case 0x062E: return "Procept"
        case 0x062F: return "ONKYO Corporation"
        case 0x0630: return "Asthrea D.O.O."
        case 0x0631: return "Fortiori Design LLC"
        case 0x0632: return "Hugo Muller GmbH & Co KG"
        case 0x0633: return "Wangi Lai PLT"
        case 0x0634: return "Fanstel Corp"
        case 0x0635: return "Crookwood"
        case 0x0636: return "ELECTRONICA INTEGRAL DE SONIDO S.A."
        case 0x0637: return "GiP Innovation Tools GmbH"
        case 0x0638: return "LX SOLUTIONS PTY LIMITED"
        case 0x0639: return "Shenzhen Minew Technologies Co., Ltd."
        case 0x063A: return "Prolojik Limited"
        case 0x063B: return "Kromek Group Plc"
        case 0x063C: return "Contec Medical Systems Co., Ltd."
        case 0x063D: return "Xradio Technology Co.,Ltd."
        case 0x063E: return "The Indoor Lab, LLC"
        case 0x063F: return "LDL TECHNOLOGY"
        case 0x0640: return "Parkifi"
        case 0x0641: return "Revenue Collection Systems FRANCE SAS"
        case 0x0642: return "Bluetrum Technology Co.,Ltd"
        case 0x0643: return "makita corporation"
        case 0x0644: return "Apogee Instruments"
        case 0x0645: return "BM3"
        case 0x0646: return "SGV Group Holding GmbH & Co. KG"
        case 0x0647: return "MED-EL"
        case 0x0648: return "Ultune Technologies"
        case 0x0649: return "Ryeex Technology Co.,Ltd."
        case 0x064A: return "Open Research Institute, Inc."
        case 0x064B: return "Scale-Tec, Ltd"
        case 0x064C: return "Zumtobel Group AG"
        case 0x064D: return "iLOQ Oy"
        case 0x064E: return "KRUXWorks Technologies Private Limited"
        case 0x064F: return "Digital Matter Pty Ltd"
        case 0x0650: return "Coravin, Inc."
        case 0x0651: return "Stasis Labs, Inc."
        case 0x0652: return "ITZ Innovations- und Technologiezentrum GmbH"
        case 0x0653: return "Meggitt SA"
        case 0x0654: return "Ledlenser GmbH & Co. KG"
        case 0x0655: return "Renishaw PLC"
        case 0x0656: return "ZhuHai AdvanPro Technology Company Limited"
        case 0x0657: return "Meshtronix Limited"
        case 0x0658: return "Payex Norge AS"
        case 0x0659: return "UnSeen Technologies Oy"
        case 0x065A: return "Zound Industries International AB"
        case 0x065B: return "Sesam Solutions BV"
        case 0x065C: return "PixArt Imaging Inc."
        case 0x065D: return "Panduit Corp."
        case 0x065E: return "Alo AB"
        case 0x065F: return "Ricoh Company Ltd"
        case 0x0660: return "RTC Industries, Inc."
        case 0x0661: return "Mode Lighting Limited"
        case 0x0662: return "Particle Industries, Inc."
        case 0x0663: return "Advanced Telemetry Systems, Inc."
        case 0x0664: return "RHA TECHNOLOGIES LTD"
        case 0x0665: return "Pure International Limited"
        case 0x0666: return "WTO Werkzeug-Einrichtungen GmbH"
        case 0x0667: return "Spark Technology Labs Inc."
        case 0x0668: return "Bleb Technology srl"
        case 0x0669: return "Livanova USA, Inc."
        case 0x066A: return "Brady Worldwide Inc."
        case 0x066B: return "DewertOkin GmbH"
        case 0x066C: return "Ztove ApS"
        case 0x066D: return "Venso EcoSolutions AB"
        case 0x066E: return "Eurotronik Kranj d.o.o."
        case 0x066F: return "Hug Technology Ltd"
        case 0x0670: return "Gema Switzerland GmbH"
        case 0x0671: return "Buzz Products Ltd."
        case 0x0672: return "Kopi"
        case 0x0673: return "Innova Ideas Limited"
        case 0x0674: return "BeSpoon"
        case 0x0675: return "Deco Enterprises, Inc."
        case 0x0676: return "Expai Solutions Private Limited"
        case 0x0677: return "Innovation First, Inc."
        case 0x0678: return "SABIK Offshore GmbH"
        case 0x0679: return "4iiii Innovations Inc."
        case 0x067A: return "The Energy Conservatory, Inc."
        case 0x067B: return "I.FARM, INC."
        case 0x067C: return "Tile, Inc."
        case 0x067D: return "Form Athletica Inc."
        case 0x067E: return "MbientLab Inc"
        case 0x067F: return "NETGRID S.N.C. DI BISSOLI MATTEO, CAMPOREALE SIMONE, TOGNETTI FEDERICO"
        case 0x0680: return "Mannkind Corporation"
        case 0x0681: return "Trade FIDES a.s."
        case 0x0682: return "Photron Limited"
        case 0x0683: return "Eltako GmbH"
        case 0x0684: return "Dermalapps, LLC"
        case 0x0685: return "Greenwald Industries"
        case 0x0686: return "inQs Co., Ltd."
        case 0x0687: return "Cherry GmbH"
        case 0x0688: return "Amsted Digital Solutions Inc."
        case 0x0689: return "Tacx b.v."
        case 0x068A: return "Raytac Corporation"
        case 0x068B: return "Jiangsu Teranovo Tech Co., Ltd."
        case 0x068C: return "Changzhou Sound Dragon Electronics and Acoustics Co., Ltd"
        case 0x068D: return "JetBeep Inc."
        case 0x068E: return "Razer Inc."
        case 0x068F: return "JRM Group Limited"
        case 0x0690: return "Eccrine Systems, Inc."
        case 0x0691: return "Curie Point AB"
        case 0x0692: return "Georg Fischer AG"
        case 0x0693: return "Hach - Danaher"
        case 0x0694: return "T&A Laboratories LLC"
        case 0x0695: return "Koki Holdings Co., Ltd."
        case 0x0696: return "Gunakar Private Limited"
        case 0x0697: return "Stemco Products Inc"
        case 0x0698: return "Wood IT Security, LLC"
        case 0x0699: return "RandomLab SAS"
        case 0x069A: return "Adero, Inc. (formerly as TrackR, Inc.)"
        case 0x069B: return "Dragonchip Limited"
        case 0x069C: return "Noomi AB"
        case 0x069D: return "Vakaros LLC"
        case 0x069E: return "Delta Electronics, Inc."
        case 0x069F: return "FlowMotion Technologies AS"
        case 0x06A0: return "OBIQ Location Technology Inc."
        case 0x06A1: return "Cardo Systems, Ltd"
        case 0x06A2: return "Globalworx GmbH"
        case 0x06A3: return "Nymbus, LLC"
        case 0x06A4: return "Sanyo Techno Solutions Tottori Co., Ltd."
        case 0x06A5: return "TEKZITEL PTY LTD"
        case 0x06A6: return "Roambee Corporation"
        case 0x06A7: return "Chipsea Technologies (ShenZhen) Corp."
        case 0x06A8: return "GD Midea Air-Conditioning Equipment Co., Ltd."
        case 0x06A9: return "Soundmax Electronics Limited"
        case 0x06AA: return "Produal Oy"
        case 0x06AB: return "HMS Industrial Networks AB"
        case 0x06AC: return "Ingchips Technology Co., Ltd."
        case 0x06AD: return "InnovaSea Systems Inc."
        case 0x06AE: return "SenseQ Inc."
        case 0x06AF: return "Shoof Technologies"
        case 0x06B0: return "BRK Brands, Inc."
        case 0x06B1: return "SimpliSafe, Inc."
        case 0x06B2: return "Tussock Innovation 2013 Limited"
        case 0x06B3: return "The Hablab ApS"
        case 0x06B4: return "Sencilion Oy"
        case 0x06B5: return "Wabilogic Ltd."
        case 0x06B6: return "Sociometric Solutions, Inc."
        case 0x06B7: return "iCOGNIZE GmbH"
        case 0x06B8: return "ShadeCraft, Inc"
        case 0x06B9: return "Beflex Inc."
        case 0x06BA: return "Beaconzone Ltd"
        case 0x06BB: return "Leaftronix Analogic Solutions Private Limited"
        case 0x06BC: return "TWS Srl"
        case 0x06BD: return "ABB Oy"
        case 0x06BE: return "HitSeed Oy"
        case 0x06BF: return "Delcom Products Inc."
        case 0x06C0: return "CAME S.p.A."
        case 0x06C1: return "Alarm.com Holdings, Inc"
        case 0x06C2: return "Measurlogic Inc."
        case 0x06C3: return "King I Electronics.Co.,Ltd"
        case 0x06C4: return "Dream Labs GmbH"
        case 0x06C5: return "Urban Compass, Inc"
        case 0x06C6: return "Simm Tronic Limited"
        case 0x06C7: return "Somatix Inc"
        case 0x06C8: return "Storz & Bickel GmbH & Co. KG"
        case 0x06C9: return "MYLAPS B.V."
        case 0x06CA: return "Shenzhen Zhongguang Infotech Technology Development Co., Ltd"
        case 0x06CB: return "Dyeware, LLC"
        case 0x06CC: return "Dongguan SmartAction Technology Co.,Ltd."
        case 0x06CD: return "DIG Corporation"
        case 0x06CE: return "FIOR & GENTZ"
        case 0x06CF: return "Belparts N.V."
        case 0x06D0: return "Etekcity Corporation"
        case 0x06D1: return "Meyer Sound Laboratories, Incorporated"
        case 0x06D2: return "CeoTronics AG"
        case 0x06D3: return "TriTeq Lock and Security, LLC"
        case 0x06D4: return "DYNAKODE TECHNOLOGY PRIVATE LIMITED"
        case 0x06D5: return "Sensirion AG"
        case 0x06D6: return "JCT Healthcare Pty Ltd"
        case 0x06D7: return "FUBA Automotive Electronics GmbH"
        case 0x06D8: return "AW Company"
        case 0x06D9: return "Shanghai Mountain View Silicon Co.,Ltd."
        case 0x06DA: return "Zliide Technologies ApS"
        case 0x06DB: return "Automatic Labs, Inc."
        case 0x06DC: return "Industrial Network Controls, LLC"
        case 0x06DD: return "Intellithings Ltd."
        case 0x06DE: return "Navcast, Inc."
        case 0x06DF: return "Hubbell Lighting, Inc."
        case 0x06E1: return "Milestone AV Technologies LLC"
        case 0x06E2: return "Alango Technologies Ltd"
        case 0x06E3: return "Spinlock Ltd"
        case 0x06E4: return "Aluna"
        case 0x06E5: return "OPTEX CO.,LTD."
        case 0x06E6: return "NIHON DENGYO KOUSAKU"
        case 0x06E7: return "VELUX A/S"
        case 0x06E8: return "Almendo Technologies GmbH"
        case 0x06E9: return "Zmartfun Electronics, Inc."
        case 0x06EA: return "SafeLine Sweden AB"
        case 0x06EB: return "Houston Radar LLC"
        case 0x06EC: return "Sigur"
        case 0x06ED: return "J Neades Ltd"
        case 0x06EE: return "Avantis Systems Limited"
        case 0x06EF: return "ALCARE Co., Ltd."
        case 0x06F0: return "Chargy Technologies, SL"
        case 0x06F1: return "Shibutani Co., Ltd."
        case 0x06F2: return "Trapper Data AB"
        case 0x06F3: return "Alfred International Inc."
        case 0x06F4: return "Near Field Solutions Ltd"
        case 0x06F5: return "Vigil Technologies Inc."
        case 0x06F6: return "Vitulo Plus BV"
        case 0x06F7: return "WILKA Schliesstechnik GmbH"
        case 0x06F8: return "BodyPlus Technology Co.,Ltd"
        case 0x06F9: return "happybrush GmbH"
        case 0x06FA: return "Enequi AB"
        case 0x06FB: return "Sartorius AG"
        case 0x06FC: return "Tom Communication Industrial Co.,Ltd."
        case 0x06FD: return "ESS Embedded System Solutions Inc."
        case 0x06FE: return "Mahr GmbH"
        case 0x06FF: return "Redpine Signals Inc"
        case 0x0700: return "TraqFreq LLC"
        case 0x0701: return "PAFERS TECH"
        case 0x0703: return "Beijing Jingdong Century Trading Co., Ltd."
        case 0x0704: return "JBX Designs Inc."
        case 0x0705: return "AB Electrolux"
        case 0x0706: return "Wernher von Braun Center for ASdvanced Research"
        case 0x0707: return "Essity Hygiene and Health Aktiebolag"
        case 0x0708: return "Be Interactive Co., Ltd"
        case 0x0709: return "Carewear Corp."
        case 0x070B: return "Element Products, Inc."
        case 0x070C: return "Beijing Winner Microelectronics Co.,Ltd"
        case 0x070D: return "SmartSnugg Pty Ltd"
        case 0x070E: return "FiveCo Sarl"
        case 0x070F: return "California Things Inc."
        case 0x0710: return "Audiodo AB"
        case 0x0711: return "ABAX AS"
        case 0x0712: return "Bull Group Company Limited"
        case 0x0713: return "Respiri Limited"
        case 0x0714: return "MindPeace Safety LLC"
        case 0x0715: return "Vgyan Solutions"
        case 0x0716: return "Altonics"
        case 0x0717: return "iQsquare BV"
        case 0x0718: return "IDIBAIX enginneering"
        case 0x0719: return "ECSG"
        case 0x071A: return "REVSMART WEARABLE HK CO LTD"
        case 0x071B: return "Precor"
        case 0x071C: return "F5 Sports, Inc"
        case 0x071D: return "exoTIC Systems"
        case 0x071E: return "DONGGUAN HELE ELECTRONICS CO., LTD"
        case 0x071F: return "Dongguan Liesheng Electronic Co.Ltd"
        case 0x0720: return "Oculeve, Inc."
        case 0x0721: return "Clover Network, Inc."
        case 0x0722: return "Xiamen Eholder Electronics Co.Ltd"
        case 0x0723: return "Ford Motor Company"
        case 0x0724: return "Guangzhou SuperSound Information Technology Co.,Ltd"
        case 0x0725: return "Tedee Sp. z o.o."
        case 0x0726: return "PHC Corporation"
        case 0x0727: return "STALKIT AS"
        case 0x0728: return "Eli Lilly and Company"
        case 0x0729: return "SwaraLink Technologies"
        case 0x072A: return "JMR embedded systems GmbH"
        case 0x072B: return "Bitkey Inc."
        case 0x072C: return "GWA Hygiene GmbH"
        case 0x072D: return "Safera Oy"
        case 0x072E: return "Open Platform Systems LLC"
        case 0x072F: return "OnePlus Electronics (Shenzhen) Co., Ltd."
        case 0x0730: return "Wildlife Acoustics, Inc."
        case 0x0731: return "ABLIC Inc."
        case 0x0732: return "Dairy Tech, Inc."
        case 0x0733: return "Iguanavation, Inc."
        case 0x0734: return "DiUS Computing Pty Ltd"
        case 0x0735: return "UpRight Technologies LTD"
        case 0x0736: return "FrancisFund, LLC"
        case 0x0737: return "LLC Navitek"
        case 0x0738: return "Glass Security Pte Ltd"
        case 0x0739: return "Jiangsu Qinheng Co., Ltd."
        case 0x073A: return "Chandler Systems Inc."
        case 0x073B: return "Fantini Cosmi s.p.a."
        case 0x073C: return "Acubit ApS"
        case 0x073D: return "Beijing Hao Heng Tian Tech Co., Ltd."
        case 0x073E: return "Bluepack S.R.L."
        case 0x073F: return "Beijing Unisoc Technologies Co., Ltd."
        case 0x0740: return "HITIQ LIMITED"
        case 0x0741: return "MAC SRL"
        case 0x0742: return "DML LLC"
        case 0x0743: return "Sanofi"
        case 0x0744: return "SOCOMEC"
        case 0x0745: return "WIZNOVA, Inc."
        case 0x0746: return "Seitec Elektronik GmbH"
        case 0x0747: return "OR Technologies Pty Ltd"
        case 0x0748: return "GuangZhou KuGou Computer Technology Co.Ltd"
        case 0x0749: return "DIAODIAO (Beijing) Technology Co., Ltd."
        case 0x074A: return "Illusory Studios LLC"
        case 0x074B: return "Sarvavid Software Solutions LLP"
        case 0x074C: return "iopool s.a."
        case 0x074D: return "Amtech Systems, LLC"
        case 0x074E: return "EAGLE DETECTION SA"
        case 0x074F: return "MEDIATECH S.R.L."
        case 0x0750: return "Hamilton Professional Services of Canada Incorporated"
        case 0x0751: return "Changsha JEMO IC Design Co.,Ltd"
        case 0x0752: return "Elatec GmbH"
        case 0x0753: return "JLG Industries, Inc."
        case 0x0754: return "Michael Parkin"
        case 0x0755: return "Brother Industries, Ltd"
        case 0x0756: return "Lumens For Less, Inc"
        case 0x0757: return "ELA Innovation"
        case 0x0758: return "umanSense AB"
        case 0x0759: return "Shanghai InGeek Cyber Security Co., Ltd."
        case 0x075A: return "HARMAN CO.,LTD."
        case 0x075B: return "Smart Sensor Devices AB"
        case 0x075C: return "Antitronics Inc."
        case 0x075D: return "RHOMBUS SYSTEMS, INC."
        case 0x075E: return "Katerra Inc."
        case 0x075F: return "Remote Solution Co., LTD."
        case 0x0760: return "Vimar SpA"
        case 0x0761: return "Mantis Tech LLC"
        case 0x0762: return "TerOpta Ltd"
        case 0x0763: return "PIKOLIN S.L."
        case 0x0764: return "WWZN Information Technology Company Limited"
        case 0x0765: return "Voxx International"
        case 0x0766: return "ART AND PROGRAM, INC."
        case 0x0767: return "NITTO DENKO ASIA TECHNICAL CENTRE PTE. LTD."
        case 0x0768: return "Peloton Interactive Inc."
        case 0x0769: return "Force Impact Technologies"
        case 0x076A: return "Dmac Mobile Developments, LLC"
        case 0x076B: return "Engineered Medical Technologies"
        case 0x076C: return "Noodle Technology inc"
        case 0x076D: return "Graesslin GmbH"
        case 0x076E: return "WuQi technologies, Inc."
        case 0x076F: return "Successful Endeavours Pty Ltd"
        case 0x0770: return "InnoCon Medical ApS"
        case 0x0771: return "Corvex Connected Safety"
        case 0x0772: return "Thirdwayv Inc."
        case 0x0773: return "Echoflex Solutions Inc."
        case 0x0774: return "C-MAX Asia Limited"
        case 0x0775: return "4eBusiness GmbH"
        case 0x0776: return "Cyber Transport Control GmbH"
        case 0x0777: return "Cue"
        case 0x0778: return "KOAMTAC INC."
        case 0x0779: return "Loopshore Oy"
        case 0x077A: return "Niruha Systems Private Limited"
        case 0x077B: return "AmaterZ, Inc."
        case 0x077C: return "radius co., ltd."
        case 0x077D: return "Sensority, s.r.o."
        case 0x077E: return "Sparkage Inc."
        case 0x077F: return "Glenview Software Corporation"
        case 0x0780: return "Finch Technologies Ltd."
        case 0x0781: return "Qingping Technology (Beijing) Co., Ltd."
        case 0x0782: return "DeviceDrive AS"
        case 0x0783: return "ESEMBER LIMITED LIABILITY COMPANY"
        case 0x0784: return "audifon GmbH & Co. KG"
        case 0x0785: return "O2 Micro, Inc."
        case 0x0786: return "HLP Controls Pty Limited"
        case 0x0787: return "Pangaea Solution"
        case 0x0788: return "BubblyNet, LLC"
        case 0x078A: return "The Wildflower Foundation"
        case 0x078B: return "Optikam Tech Inc."
        case 0x078C: return "MINIBREW HOLDING B.V"
        case 0x078D: return "Cybex GmbH"
        case 0x078E: return "FUJIMIC NIIGATA, INC."
        case 0x078F: return "Hanna Instruments, Inc."
        case 0x0790: return "KOMPAN A/S"
        case 0x0791: return "Scosche Industries, Inc."
        case 0x0792: return "Provo Craft"
        case 0x0793: return "AEV spol. s r.o."
        case 0x0794: return "The Coca-Cola Company"
        case 0x0795: return "GASTEC CORPORATION"
        case 0x0796: return "StarLeaf Ltd"
        case 0x0797: return "Water-i.d. GmbH"
        case 0x0798: return "HoloKit, Inc."
        case 0x0799: return "PlantChoir Inc."
        case 0x079A: return "GuangDong Oppo Mobile Telecommunications Corp., Ltd."
        case 0x079B: return "CST ELECTRONICS (PROPRIETARY) LIMITED"
        case 0x079C: return "Sky UK Limited"
        case 0x079D: return "Digibale Pty Ltd"
        case 0x079E: return "Smartloxx GmbH"
        case 0x079F: return "Pune Scientific LLP"
        case 0x07A0: return "Regent Beleuchtungskorper AG"
        case 0x07A1: return "Apollo Neuroscience, Inc."
        case 0x07A2: return "Roku, Inc."
        case 0x07A3: return "Comcast Cable"
        case 0x07A4: return "Xiamen Mage Information Technology Co., Ltd."
        case 0x07A5: return "RAB Lighting, Inc."
        case 0x07A6: return "Musen Connect, Inc."
        case 0x07A7: return "Zume, Inc."
        case 0x07A8: return "conbee GmbH"
        case 0x07A9: return "Bruel & Kjaer Sound & Vibration"
        case 0x07AA: return "The Kroger Co."
        case 0x07AB: return "Granite River Solutions, Inc."
        case 0x07AC: return "LoupeDeck Oy"
        case 0x07AD: return "New H3C Technologies Co.,Ltd"
        case 0x07AE: return "Aurea Solucoes Tecnologicas Ltda."
        case 0x07AF: return "Hong Kong Bouffalo Lab Limited"
        case 0x07B0: return "GV Concepts Inc."
        case 0x07B1: return "Thomas Dynamics, LLC"
        case 0x07B2: return "Moeco IOT Inc."
        case 0x07B3: return "2N TELEKOMUNIKACE a.s."
        case 0x07B4: return "Hormann KG Antriebstechnik"
        case 0x07B5: return "CRONO CHIP, S.L."
        case 0x07B6: return "Soundbrenner Limited"
        case 0x07B7: return "ETABLISSEMENTS GEORGES RENAULT"
        case 0x07B8: return "iSwip"
        case 0x07B9: return "Epona Biotec Limited"
        case 0x07BA: return "Battery-Biz Inc."
        case 0x07BB: return "EPIC S.R.L."
        case 0x07BC: return "KD CIRCUITS LLC"
        case 0x07BD: return "Genedrive Diagnostics Ltd"
        case 0x07BE: return "Axentia Technologies AB"
        case 0x07BF: return "REGULA Ltd."
        case 0x07C0: return "Biral AG"
        case 0x07C1: return "A.W. Chesterton Company"
        case 0x07C2: return "Radinn AB"
        case 0x07C3: return "CIMTechniques, Inc."
        case 0x07C4: return "Johnson Health Tech NA"
        case 0x07C5: return "June Life, Inc."
        case 0x07C6: return "Bluenetics GmbH"
        case 0x07C7: return "iaconicDesign Inc."
        case 0x07C8: return "WRLDS Creations AB"
        case 0x07C9: return "Skullcandy, Inc."
        case 0x07CA: return "Modul-System HH AB"
        case 0x07CB: return "West Pharmaceutical Services, Inc."
        case 0x07CC: return "Barnacle Systems Inc."
        case 0x07CD: return "Smart Wave Technologies Canada Inc"
        case 0x07CE: return "Shanghai Top-Chip Microelectronics Tech. Co., LTD"
        case 0x07CF: return "NeoSensory, Inc."
        case 0x07D0: return "Hangzhou Tuya Information Technology Co., Ltd"
        case 0x07D1: return "Shanghai Panchip Microelectronics Co., Ltd"
        case 0x07D2: return "React Accessibility Limited"
        case 0x07D3: return "LIVNEX Co.,Ltd."
        case 0x07D4: return "Kano Computing Limited"
        case 0x07D5: return "hoots classic GmbH"
        case 0x07D6: return "ecobee Inc."
        case 0x07D7: return "Nanjing Qinheng Microelectronics Co., Ltd"
        case 0x07D8: return "SOLUTIONS AMBRA INC."
        case 0x07D9: return "Micro-Design, Inc."
        case 0x07DA: return "STARLITE Co., Ltd."
        case 0x07DB: return "Remedee Labs"
        case 0x07DC: return "ThingOS GmbH"
        case 0x07DD: return "Linear Circuits"
        case 0x07DE: return "Unlimited Engineering SL"
        case 0x07DF: return "Snap-on Incorporated"
        case 0x07E0: return "Edifier International Limited"
        case 0x07E1: return "Lucie Labs"
        case 0x07E2: return "Alfred Kaercher SE & Co. KG"
        case 0x07E3: return "Audiowise Technology Inc."
        case 0x07E4: return "Geeksme S.L."
        case 0x07E5: return "Minut, Inc."
        case 0x07E6: return "Autogrow Systems Limited"
        case 0x07E7: return "Komfort IQ, Inc."
        case 0x07E8: return "Packetcraft, Inc."
        case 0x07E9: return "Häfele GmbH & Co KG"
        case 0x07EA: return "ShapeLog, Inc."
        case 0x07EB: return "NOVABASE S.R.L."
        case 0x07EC: return "Frecce LLC"
        case 0x07ED: return "Joule IQ, INC."
        case 0x07EE: return "KidzTek LLC"
        case 0x07EF: return "Aktiebolaget Sandvik Coromant"
        case 0x07F0: return "e-moola.com Pty Ltd"
        case 0x07F1: return "GSM Innovations Pty Ltd"
        case 0x07F2: return "SERENE GROUP, INC"
        case 0x07F3: return "DIGISINE ENERGYTECH CO. LTD."
        case 0x07F4: return "MEDIRLAB Orvosbiologiai Fejleszto Korlatolt Felelossegu Tarsasag"
        case 0x07F5: return "Byton North America Corporation"
        case 0x07F6: return "Shenzhen TonliScience and Technology Development Co.,Ltd"
        case 0x07F7: return "Cesar Systems Ltd."
        case 0x07F8: return "quip NYC Inc."
        case 0x07F9: return "Direct Communication Solutions, Inc."
        case 0x07FA: return "Klipsch Group, Inc."
        case 0x07FB: return "Access Co., Ltd"
        case 0x07FC: return "Renault SA"
        case 0x07FD: return "JSK CO., LTD."
        case 0x07FE: return "BIROTA"
        case 0x07FF: return "maxon motor ltd."
        case 0x0800: return "Optek"
        case 0x0801: return "CRONUS ELECTRONICS LTD"
        case 0x0802: return "NantSound, Inc."
        case 0x0803: return "Domintell s.a."
        case 0x0804: return "Andon Health Co.,Ltd"
        case 0x0805: return "Urbanminded Ltd"
        case 0x0806: return "TYRI Sweden AB"
        case 0x0807: return "ECD Electronic Components GmbH Dresden"
        case 0x0808: return "SISTEMAS KERN, SOCIEDAD ANÓMINA"
        case 0x0809: return "Trulli Audio"
        case 0x080A: return "Altaneos"
        case 0x080B: return "Nanoleaf Canada Limited"
        case 0x080C: return "Ingy B.V."
        case 0x080D: return "Azbil Co."
        case 0x080E: return "TATTCOM LLC"
        case 0x080F: return "Paradox Engineering SA"
        case 0x0810: return "LECO Corporation"
        case 0x0811: return "Becker Antriebe GmbH"
        case 0x0812: return "Mstream Technologies., Inc."
        case 0x0813: return "Flextronics International USA Inc."
        case 0x0814: return "Ossur hf."
        case 0x0815: return "SKC Inc"
        case 0x0816: return "SPICA SYSTEMS LLC"
        case 0x0817: return "Wangs Alliance Corporation"
        case 0x0818: return "tatwah SA"
        case 0x0819: return "Hunter Douglas Inc"
        case 0x081A: return "Shenzhen Conex"
        case 0x081B: return "DIM3"
        case 0x081C: return "Bobrick Washroom Equipment, Inc."
        case 0x081D: return "Potrykus Holdings and Development LLC"
        case 0x081E: return "iNFORM Technology GmbH"
        case 0x081F: return "eSenseLab LTD"
        case 0x0820: return "Brilliant Home Technology, Inc."
        case 0x0821: return "INOVA Geophysical, Inc."
        case 0x0822: return "adafruit industries"
        case 0x0823: return "Nexite Ltd"
        case 0x0824: return "8Power Limited"
        case 0x0825: return "CME PTE. LTD."
        case 0x0826: return "Hyundai Motor Company"
        case 0x0827: return "Kickmaker"
        case 0x0828: return "Shanghai Suisheng Information Technology Co., Ltd."
        case 0x0829: return "HEXAGON"
        case 0x082A: return "Mitutoyo Corporation"
        case 0x082B: return "shenzhen fitcare electronics Co.,Ltd"
        case 0x082C: return "INGICS TECHNOLOGY CO., LTD."
        case 0x082D: return "INCUS PERFORMANCE LTD."
        case 0x082E: return "ABB S.p.A."
        case 0x082F: return "Blippit AB"
        case 0x0830: return "Core Health and Fitness LLC"
        case 0x0831: return "Foxble, LLC"
        case 0x0832: return "Intermotive,Inc."
        case 0x0833: return "Conneqtech B.V."
        case 0x0834: return "RIKEN KEIKI CO., LTD.,"
        case 0x0835: return "Canopy Growth Corporation"
        case 0x0836: return "Bitwards Oy"
        case 0x0837: return "vivo Mobile Communication Co., Ltd."
        case 0x0838: return "Etymotic Research, Inc."
        case 0x0839: return "A puissance 3"
        case 0x083A: return "BPW Bergische Achsen Kommanditgesellschaft"
        case 0x083B: return "Piaggio Fast Forward"
        case 0x083C: return "BeerTech LTD"
        case 0x083D: return "Tokenize, Inc."
        case 0x083E: return "Zorachka LTD"
        case 0x083F: return "D-Link Corp."
        case 0x0840: return "Down Range Systems LLC"
        case 0x0841: return "General Luminaire (Shanghai) Co., Ltd."
        case 0x0842: return "Tangshan HongJia electronic technology co., LTD."
        case 0x0843: return "FRAGRANCE DELIVERY TECHNOLOGIES LTD"
        case 0x0844: return "Pepperl + Fuchs GmbH"
        case 0x0845: return "Dometic Corporation"
        case 0x0846: return "USound GmbH"
        case 0x0847: return "DNANUDGE LIMITED"
        case 0x0848: return "JUJU JOINTS CANADA CORP."
        case 0x0849: return "Dopple Technologies B.V."
        case 0x084A: return "ARCOM"
        case 0x084B: return "Biotechware SRL"
        case 0x084C: return "ORSO Inc."
        case 0x084D: return "SafePort"
        case 0x084E: return "Carol Cole Company"
        case 0x084F: return "Embedded Fitness B.V."
        case 0x0850: return "Yealink (Xiamen) Network Technology Co.,LTD"
        case 0x0851: return "Subeca, Inc."
        case 0x0852: return "Cognosos, Inc."
        case 0x0853: return "Pektron Group Limited"
        case 0x0854: return "Tap Sound System"
        case 0x0855: return "Helios Hockey, Inc."
        case 0x0856: return "Canopy Growth Corporation"
        case 0x0857: return "Parsyl Inc"
        case 0x0858: return "SOUNDBOKS"
        case 0x0859: return "BlueUp"
        case 0x085A: return "DAKATECH"
        case 0x085B: return "RICOH ELECTRONIC DEVICES CO., LTD."
        case 0x085C: return "ACOS CO.,LTD."
        case 0x085D: return "Guilin Zhishen Information Technology Co.,Ltd."
        case 0x085E: return "Krog Systems LLC"
        case 0x085F: return "COMPEGPS TEAM,SOCIEDAD LIMITADA"
        case 0x0860: return "Alflex Products B.V."
        case 0x0861: return "SmartSensor Labs Ltd"
        case 0x0862: return "SmartDrive Inc."
        case 0x0863: return "Yo-tronics Technology Co., Ltd."
        case 0x0864: return "Rafaelmicro"
        case 0x0865: return "Emergency Lighting Products Limited"
        case 0x0866: return "LAONZ Co.,Ltd"
        case 0x0867: return "Western Digital Techologies, Inc."
        case 0x0868: return "WIOsense GmbH & Co. KG"
        case 0x0869: return "EVVA Sicherheitstechnologie GmbH"
        case 0x086A: return "Odic Incorporated"
        case 0x086B: return "Pacific Track, LLC"
        case 0x086C: return "Revvo Technologies, Inc."
        case 0x086D: return "Biometrika d.o.o."
        case 0x086E: return "Vorwerk Elektrowerke GmbH & Co. KG"
        case 0x086F: return "Trackunit A/S"
        case 0x0870: return "Wyze Labs, Inc"
        case 0x0871: return "Dension Elektronikai Kft. (formerly: Dension Audio Systems Ltd.)"
        case 0x0872: return "11 Health & Technologies Limited"
        case 0x0873: return "Innophase Incorporated"
        case 0x0874: return "Treegreen Limited"
        case 0x0875: return "Berner International LLC"
        case 0x0876: return "SmartResQ ApS"
        case 0x0877: return "Tome, Inc."
        case 0x0878: return "The Chamberlain Group, Inc."
        case 0x0879: return "MIZUNO Corporation"
        case 0x087A: return "ZRF, LLC"
        case 0x087B: return "BYSTAMP"
        case 0x087C: return "Crosscan GmbH"
        case 0x087D: return "Konftel AB"
        case 0x087E: return "1bar.net Limited"
        case 0x087F: return "Phillips Connect Technologies LLC"
        case 0x0880: return "imagiLabs AB"
        case 0x0881: return "Optalert"
        case 0x0882: return "PSYONIC, Inc."
        case 0x0883: return "Wintersteiger AG"
        case 0x0884: return "Controlid Industria, Comercio de Hardware e Servicos de Tecnologia Ltda"
        case 0x0885: return "LEVOLOR, INC."
        case 0x0886: return "Xsens Technologies B.V."
        case 0x0887: return "Hydro-Gear Limited Partnership"
        case 0x0888: return "EnPointe Fencing Pty Ltd"
        case 0x0889: return "XANTHIO"
        case 0x088A: return "sclak s.r.l."
        case 0x088B: return "Tricorder Arraay Technologies LLC"
        case 0x088C: return "GB Solution co.,Ltd"
        case 0x088D: return "Soliton Systems K.K."
        case 0x088E: return "GIGA-TMS INC"
        case 0x088F: return "Tait International Limited"
        case 0x0890: return "NICHIEI INTEC CO., LTD."
        case 0x0891: return "SmartWireless GmbH & Co. KG"
        case 0x0892: return "Ingenieurbuero Birnfeld UG (haftungsbeschraenkt)"
        case 0x0893: return "Maytronics Ltd"
        case 0x0894: return "EPIFIT"
        case 0x0895: return "Gimer medical"
        case 0x0896: return "Nokian Renkaat Oyj"
        case 0x0897: return "Current Lighting Solutions LLC"
        case 0x0898: return "Sensibo, Inc."
        case 0x0899: return "SFS unimarket AG"
        case 0x089A: return "Private limited company \"Teltonika\""
        case 0x089B: return "Saucon Technologies"
        case 0x089C: return "Embedded Devices Co. Company"
        case 0x089D: return "J-J.A.D.E. Enterprise LLC"
        case 0x089E: return "i-SENS, inc."
        case 0x089F: return "Witschi Electronic Ltd"
        case 0x08A0: return "Aclara Technologies LLC"
        case 0x08A1: return "EXEO TECH CORPORATION"
        case 0x08A2: return "Epic Systems Co., Ltd."
        case 0x08A3: return "Hoffmann SE"
        case 0x08A4: return "Realme Chongqing Mobile Telecommunications Corp., Ltd."
        case 0x08A5: return "UMEHEAL Ltd"
        case 0x08A6: return "Intelligenceworks Inc."
        case 0x08A7: return "TGR 1.618 Limited"
        case 0x08A8: return "Shanghai Kfcube Inc"
        case 0x08A9: return "Fraunhofer IIS"
        case 0x08AA: return "SZ DJI TECHNOLOGY CO.,LTD"
        case 0x08AB: return "Coburn Technology, LLC"
        case 0x08AC: return "Topre Corporation"
        case 0x08AD: return "Kayamatics Limited"
        case 0x08AE: return "Moticon ReGo AG"
        case 0x08AF: return "Polidea Sp. z o.o."
        case 0x08B0: return "Trivedi Advanced Technologies LLC"
        case 0x08B1: return "CORE|vision BV"
        case 0x08B2: return "PF SCHWEISSTECHNOLOGIE GMBH"
        case 0x08B3: return "IONIQ Skincare GmbH & Co. KG"
        case 0x08B4: return "Sengled Co., Ltd."
        case 0x08B5: return "TransferFi"
        case 0x08B6: return "Boehringer Ingelheim Vetmedica GmbH"
        case 0x08B7: return "ABB Inc"
        case 0x08B8: return "Check Technology Solutions LLC"
        case 0x08B9: return "U-Shin Ltd."
        case 0x08BA: return "HYPER ICE, INC."
        case 0x08BB: return "Tokai-rika co.,ltd."
        case 0x08BC: return "Prevayl Limited"
        case 0x08BD: return "bf1systems limited"
        case 0x08BE: return "ubisys technologies GmbH"
        case 0x08BF: return "SIRC Co., Ltd."
        case 0x08C0: return "Accent Advanced Systems SLU"
        case 0x08C1: return "Rayden.Earth LTD"
        case 0x08C2: return "Lindinvent AB"
        case 0x08C3: return "CHIPOLO d.o.o."
        case 0x08C4: return "CellAssist, LLC"
        case 0x08C5: return "J. Wagner GmbH"
        case 0x08C6: return "Integra Optics Inc"
        case 0x08C7: return "Monadnock Systems Ltd."
        case 0x08C8: return "Liteboxer Technologies Inc."
        case 0x08C9: return "Noventa AG"
        case 0x08CA: return "Nubia Technology Co.,Ltd."
        case 0x08CB: return "JT INNOVATIONS LIMITED"
        case 0x08CC: return "TGM TECHNOLOGY CO., LTD."
        case 0x08CD: return "ifly"
        case 0x08CE: return "ZIMI CORPORATION"
        case 0x08CF: return "betternotstealmybike UG (with limited liability)"
        case 0x08D0: return "ESTOM Infotech Kft."
        case 0x08D1: return "Sensovium Inc."
        case 0x08D2: return "Virscient Limited"
        case 0x08D3: return "Novel Bits, LLC"
        case 0x08D4: return "ADATA Technology Co., LTD."
        case 0x08D5: return "KEYes"
        case 0x08D6: return "Nome Oy"
        case 0x08D7: return "Inovonics Corp"
        case 0x08D8: return "WARES"
        case 0x08D9: return "Pointr Labs Limited"
        case 0x08DA: return "Miridia Technology Incorporated"
        case 0x08DB: return "Tertium Technology"
        case 0x08DC: return "SHENZHEN AUKEY E BUSINESS CO., LTD"
        case 0x08DD: return "code-Q"
        case 0x08DE: return "Tyco Electronics Corporation a TE Connectivity Ltd Company"
        case 0x08DF: return "IRIS OHYAMA CO.,LTD."
        case 0x08E0: return "Philia Technology"
        case 0x08E1: return "KOZO KEIKAKU ENGINEERING Inc."
        case 0x08E2: return "Shenzhen Simo Technology co. LTD"
        case 0x08E3: return "Republic Wireless, Inc."
        case 0x08E4: return "Rashidov ltd"
        case 0x08E5: return "Crowd Connected Ltd"
        case 0x08E6: return "Eneso Tecnologia de Adaptacion S.L."
        case 0x08E7: return "Barrot Technology Limited"
        case 0x08E8: return "Naonext"
        case 0x08E9: return "Taiwan Intelligent Home Corp."
        case 0x08EA: return "COWBELL ENGINEERING CO.,LTD."
        case 0x08EB: return "Beijing Big Moment Technology Co., Ltd."
        case 0x08EC: return "Denso Corporation"
        case 0x08ED: return "IMI Hydronic Engineering International SA"
        case 0x08EE: return "ASKEY"
        case 0x08EF: return "Cumulus Digital Systems, Inc"
        case 0x08F0: return "Joovv, Inc."
        case 0x08F1: return "The L.S. Starrett Company"
        case 0x08F2: return "Microoled"
        case 0x08F3: return "PSP - Pauli Services & Products GmbH"
        case 0x08F4: return "Kodimo Technologies Company Limited"
        case 0x08F5: return "Tymtix Technologies Private Limited"
        case 0x08F6: return "Dermal Photonics Corporation"
        case 0x08F7: return "MTD Products Inc & Affiliates"
        case 0x08F8: return "instagrid GmbH"
        case 0x08F9: return "Spacelabs Medical Inc."
        case 0x08FA: return "Troo Corporation"
        case 0x08FB: return "Darkglass Electronics Oy"
        case 0x08FC: return "Hill-Rom"
        case 0x08FD: return "BioIntelliSense, Inc."
        case 0x08FE: return "Ketronixs Sdn Bhd"
        case 0x0904: return "SUNCORPORATION"
        case 0x0905: return "Yandex Services AG"
        case 0x0906: return "Scope Logistical Solutions"
        case 0x0907: return "User Hello, LLC"
        case 0x0908: return "Pinpoint Innovations Limited"
        case 0x0909: return "70mai Co.,Ltd."
        case 0x090A: return "Zhuhai Hoksi Technology CO.,LTD"
        case 0x090B: return "EMBR labs, INC"
        case 0x090C: return "Radiawave Technologies Co.,Ltd."
        case 0x090D: return "IOT Invent GmbH"
        case 0x090E: return "OPTIMUSIOT TECH LLP"
        case 0x090F: return "VC Inc."
        case 0x0910: return "ASR Microelectronics (Shanghai) Co., Ltd."
        case 0x0911: return "Douglas Lighting Controls Inc."
        case 0x0912: return "Nerbio Medical Software Platforms Inc"
        case 0x0913: return "Braveheart Wireless, Inc."
        case 0x0914: return "INEO-SENSE"
        case 0x0915: return "Honda Motor Co., Ltd."
        case 0x0916: return "Ambient Sensors LLC"
        case 0x0917: return "ASR Microelectronics(ShenZhen)Co., Ltd."
        case 0x0918: return "Technosphere Labs Pvt. Ltd."
        case 0x0919: return "NO SMD LIMITED"
        case 0x091A: return "Albertronic BV"
        case 0x091B: return "Luminostics, Inc."
        case 0x091C: return "Oblamatik AG"
        case 0x091D: return "Innokind, Inc."
        case 0x091E: return "Melbot Studios, Sociedad Limitada"
        case 0x091F: return "Myzee Technology"
        case 0x0920: return "Omnisense Limited"
        case 0x0921: return "KAHA PTE. LTD."
        case 0x0922: return "Shanghai MXCHIP Information Technology Co., Ltd."
        case 0x0923: return "JSB TECH PTE LTD"
        case 0x0924: return "Fundacion Tecnalia Research and Innovation"
        case 0x0925: return "Yukai Engineering Inc."
        case 0x0926: return "Gooligum Technologies Pty Ltd"
        case 0x0927: return "ROOQ GmbH"
        case 0x0928: return "AiRISTA"
        case 0x0929: return "Qingdao Haier Technology Co., Ltd."
        case 0x092A: return "Sappl Verwaltungs- und Betriebs GmbH"
        case 0x092B: return "TekHome"
        case 0x092C: return "PCI Private Limited"
        case 0x092D: return "Leggett & Platt, Incorporated"
        case 0x092E: return "PS GmbH"
        case 0x092F: return "C.O.B.O. SpA"
        case 0x0930: return "James Walker RotaBolt Limited"
        case 0x0931: return "BREATHINGS Co., Ltd."
        case 0x0932: return "BarVision, LLC"
        case 0x0933: return "SRAM"
        case 0x0934: return "KiteSpring Inc."
        case 0x0935: return "Reconnect, Inc."
        case 0x0936: return "Elekon AG"
        case 0x0937: return "RealThingks GmbH"
        case 0x0938: return "Henway Technologies, LTD."
        case 0x0939: return "ASTEM Co.,Ltd."
        case 0x093A: return "LinkedSemi Microelectronics (Xiamen) Co., Ltd"
        case 0x093B: return "ENSESO LLC"
        case 0x093C: return "Xenoma Inc."
        case 0x093D: return "Adolf Wuerth GmbH & Co KG"
        case 0x093E: return "Catalyft Labs, Inc."
        case 0x093F: return "JEPICO Corporation"
        case 0x0940: return "Hero Workout GmbH"
        case 0x0941: return "Rivian Automotive, LLC"
        case 0x0942: return "TRANSSION HOLDINGS LIMITED"
        case 0x0943: return "Inovonics Corp."
        case 0x0944: return "Agitron d.o.o."
        case 0x0945: return "Globe (Jiangsu) Co., Ltd"
        case 0x0946: return "AMC International Alfa Metalcraft Corporation AG"
        case 0x0947: return "First Light Technologies Ltd."
        case 0x0948: return "Wearable Link Limited"
        case 0x0949: return "Metronom Health Europe"
        case 0x094A: return "Zwift, Inc."
        case 0x094B: return "Kindeva Drug Delivery L.P."
        case 0x094C: return "GimmiSys GmbH"
        case 0x094D: return "tkLABS INC."
        case 0x094E: return "PassiveBolt, Inc."
        case 0x094F: return "Limited Liability Company \"Mikrotikls\""
        case 0x0950: return "Capetech"
        case 0x0951: return "PPRS"
        case 0x0952: return "Apptricity Corporation"
        case 0x0953: return "LogiLube, LLC"
        case 0x0954: return "Julbo"
        case 0x0955: return "Breville Group"
        case 0x0956: return "Kerlink"
        case 0x0957: return "Ohsung Electronics"
        case 0x0958: return "ZTE Corporation"
        case 0x0959: return "HerdDogg, Inc"
        case 0x095A: return "Selekt Bilgisayar, lletisim Urunleri lnsaat Sanayi ve Ticaret Limited Sirketi"
        case 0x095B: return "Lismore Instruments Limited"
        case 0x095C: return "LogiLube, LLC"
        case 0x095D: return "ETC"
        case 0x095E: return "BioEchoNet inc."
        case 0x095F: return "NUANCE HEARING LTD"
        case 0x0960: return "Sena Technologies Inc."
        case 0x0961: return "Linkura AB"
        case 0x0962: return "GL Solutions K.K."
        case 0x0963: return "Moonbird BV"
        case 0x0964: return "Countrymate Technology Limited"
        case 0x0965: return "Asahi Kasei Corporation"
        case 0x0966: return "PointGuard, LLC"
        case 0x0967: return "Neo Materials and Consulting Inc."
        case 0x0968: return "Actev Motors, Inc."
        case 0x0969: return "Woan Technology (Shenzhen) Co., Ltd."
        case 0x096A: return "dricos, Inc."
        case 0x096B: return "Guide ID B.V."
        case 0x096C: return "9374-7319 Quebec inc"
        case 0x096D: return "Gunwerks, LLC"
        case 0x096E: return "Band Industries, inc."
        case 0x096F: return "Lund Motion Products, Inc."
        case 0x0970: return "IBA Dosimetry GmbH"
        case 0x0971: return "GA"
        case 0x0972: return "Closed Joint Stock Company \"Zavod Flometr\" (\"Zavod Flometr\" CJSC)"
        case 0x0973: return "Popit Oy"
        case 0x0974: return "ABEYE"
        case 0x0975: return "BlueIOT(Beijing) Technology Co.,Ltd"
        case 0x0976: return "Fauna Audio GmbH"
        case 0x0977: return "TOYOTA motor corporation"
        case 0x0978: return "ZifferEins GmbH & Co. KG"
        case 0x0979: return "BIOTRONIK SE & Co. KG"
        case 0x097A: return "CORE CORPORATION"
        case 0x097B: return "CTEK Sweden AB"
        case 0x097C: return "Thorley Industries, LLC"
        case 0x097D: return "CLB B.V."
        case 0x097E: return "SonicSensory Inc"
        case 0x097F: return "ISEMAR S.R.L."
        case 0x0980: return "DEKRA TESTING AND CERTIFICATION, S.A.U."
        case 0x0981: return "Bernard Krone Holding SE & Co.KG"
        case 0x0982: return "ELPRO-BUCHS AG"
        case 0x0983: return "Feedback Sports LLC"
        case 0x0984: return "TeraTron GmbH"
        case 0x0985: return "Lumos Health Inc."
        case 0x0986: return "Cello Hill, LLC"
        case 0x0987: return "TSE BRAKES, INC."
        case 0x0988: return "BHM-Tech Produktionsgesellschaft m.b.H"
        case 0x0989: return "WIKA Alexander Wiegand SE & Co.KG"
        case 0x098A: return "Biovigil"
        case 0x098B: return "Mequonic Engineering, S.L."
        case 0x098C: return "bGrid B.V."
        case 0x098D: return "C3-WIRELESS, LLC"
        case 0x098E: return "ADVEEZ"
        case 0x098F: return "Aktiebolaget Regin"
        case 0x0990: return "Anton Paar GmbH"
        case 0x0991: return "Telenor ASA"
        case 0x0992: return "Big Kaiser Precision Tooling Ltd"
        case 0x0993: return "Absolute Audio Labs B.V."
        case 0x0994: return "VT42 Pty Ltd"
        case 0x0995: return "Bronkhorst High-Tech B.V."
        case 0x0996: return "C. & E. Fein GmbH"
        case 0x0997: return "NextMind"
        case 0x0998: return "Pixie Dust Technologies, Inc."
        case 0x0999: return "eTactica ehf"
        case 0x099A: return "New Audio LLC"
        case 0x099B: return "Sendum Wireless Corporation"
        case 0x099C: return "deister electronic GmbH"
        case 0x099D: return "YKK AP Inc."
        case 0x099E: return "Step One Limited"
        case 0x099F: return "Koya Medical, Inc."
        case 0x09A0: return "Pine Trees Health, Inc"
        case 0x09A1: return "VOS Systems, LLC"
        case 0x09A2: return "ENGAGENOW DATA SCIENCES PRIVATE LIMITED"
        case 0x09A3: return "ARDUINO SA"
        case 0x09A4: return "KUMHO ELECTRICS, INC"
        case 0x09A5: return "Security Enhancement Systems, LLC"
        case 0x09A6: return "BEIJING ELECTRIC VEHICLE CO.,LTD"
        case 0x09A7: return "Paybuddy ApS"
        case 0x09A8: return "KHN Solutions Inc"
        case 0x09A9: return "Nippon Ceramic Co.,Ltd."
        case 0x09AA: return "PHOTODYNAMIC INCORPORATED"
        case 0x09AB: return "DashLogic, Inc."
        case 0x09AC: return "Ambiq"
        case 0x09AD: return "Narhwall Inc."
        case 0x09AE: return "Pozyx NV"
        case 0x09AF: return "ifLink Open Community"
        case 0x09B0: return "Deublin Company, LLC"
        case 0x09B1: return "BLINQY"
        case 0x09B2: return "DYPHI"
        case 0x09B3: return "BlueX Microelectronics Corp Ltd."
        case 0x09B4: return "PentaLock Aps."
        case 0x09B5: return "AUTEC Gesellschaft fuer Automationstechnik mbH"
        case 0x09B6: return "Pegasus Technologies, Inc."
        case 0x09B7: return "Bout Labs, LLC"
        case 0x09B8: return "PlayerData Limited"
        case 0x09B9: return "SAVOY ELECTRONIC LIGHTING"
        case 0x09BA: return "Elimo Engineering Ltd"
        case 0x09BB: return "SkyStream Corporation"
        case 0x09BC: return "Aerosens LLC"
        case 0x09BD: return "Centre Suisse d'Electronique et de Microtechnique SA"
        case 0x09BE: return "Vessel Ltd."
        case 0x09BF: return "Span.IO, Inc."
        case 0x09C0: return "AnotherBrain inc."
        case 0x09C1: return "Rosewill"
        case 0x09C2: return "Universal Audio, Inc."
        case 0x09C3: return "JAPAN TOBACCO INC."
        case 0x09C4: return "UVISIO"
        case 0x09C5: return "HungYi Microelectronics Co.,Ltd."
        case 0x09C6: return "Honor Device Co., Ltd."
        case 0x09C7: return "Combustion, LLC"
        case 0x09C8: return "XUNTONG"
        case 0x09C9: return "CrowdGlow Ltd"
        case 0x09CA: return "Mobitrace"
        case 0x09CB: return "Hx Engineering, LLC"
        case 0x09CC: return "Senso4s d.o.o."
        case 0x09CD: return "Blyott"
        case 0x09CE: return "Julius Blum GmbH"
        case 0x09CF: return "BlueStreak IoT, LLC"
        case 0x09D0: return "Chess Wise B.V."
        case 0x09D1: return "ABLEPAY TECHNOLOGIES AS"
        case 0x09D2: return "Temperature Sensitive Solutions Systems Sweden AB"
        case 0x09D3: return "HeartHero, inc."
        case 0x09D4: return "ORBIS Inc."
        case 0x09D5: return "GEAR RADIO ELECTRONICS CORP."
        case 0x09D6: return "EAR TEKNIK ISITME VE ODIOMETRI CIHAZLARI SANAYI VE TICARET ANONIM SIRKETI"
        case 0x09D7: return "Coyotta"
        case 0x09D8: return "Synergy Tecnologia em Sistemas Ltda"
        case 0x09D9: return "VivoSensMedical GmbH"
        case 0x09DA: return "Nagravision SA"
        case 0x09DB: return "Bionic Avionics Inc."
        case 0x09DC: return "AON2 Ltd."
        case 0x09DD: return "Innoware Development AB"
        case 0x09DE: return "JLD Technology Solutions, LLC"
        case 0x09DF: return "Magnus Technology Sdn Bhd"
        case 0x09E0: return "Preddio Technologies Inc."
        case 0x09E1: return "Tag-N-Trac Inc"
        case 0x09E2: return "Wuhan Linptech Co.,Ltd."
        case 0x09E3: return "Friday Home Aps"
        case 0x09E4: return "CPS AS"
        case 0x09E5: return "Mobilogix"
        case 0x09E6: return "Masonite Corporation"
        case 0x09E7: return "Kabushikigaisha HANERON"
        case 0x09E8: return "Melange Systems Pvt. Ltd."
        case 0x09E9: return "LumenRadio AB"
        case 0x09EA: return "Athlos Oy"
        case 0x09EB: return "KEAN ELECTRONICS PTY LTD"
        case 0x09EC: return "Yukon advanced optics worldwide, UAB"
        case 0x09ED: return "Sibel Inc."
        case 0x09EE: return "OJMAR SA"
        case 0x09EF: return "Steinel Solutions AG"
        case 0x09F0: return "WatchGas B.V."
        case 0x09F1: return "OM Digital Solutions Corporation"
        case 0x09F2: return "Audeara Pty Ltd"
        case 0x09F3: return "Beijing Zero Zero Infinity Technology Co.,Ltd."
        case 0x09F4: return "Spectrum Technologies, Inc."
        case 0x09F5: return "OKI Electric Industry Co., Ltd"
        case 0x09F6: return "Mobile Action Technology Inc."
        case 0x09F7: return "SENSATEC Co., Ltd."
        case 0x09F8: return "R.O. S.R.L."
        case 0x09F9: return "Hangzhou Yaguan Technology Co. LTD"
        case 0x09FA: return "Listen Technologies Corporation"
        case 0x09FB: return "TOITU CO., LTD."
        case 0x09FC: return "Confidex"
        case 0x09FD: return "Keep Technologies, Inc."
        case 0x09FE: return "Lichtvision Engineering GmbH"
        case 0x09FF: return "AIRSTAR"
        case 0x0A00: return "Ampler Bikes OU"
        case 0x0A01: return "Cleveron AS"
        case 0x0A02: return "Ayxon-Dynamics GmbH"
        case 0x0A03: return "donutrobotics Co., Ltd."
        case 0x0A04: return "Flosonics Medical"
        case 0x0A05: return "Southwire Company, LLC"
        case 0x0A06: return "Shanghai wuqi microelectronics Co.,Ltd"
        case 0x0A07: return "Reflow Pty Ltd"
        case 0x0A08: return "Oras Oy"
        case 0x0A09: return "ECCT"
        case 0x0A0A: return "Volan Technology Inc."
        case 0x0A0B: return "SIANA Systems"
        case 0x0A0C: return "Shanghai Yidian Intelligent Technology Co., Ltd."
        case 0x0A0D: return "Blue Peacock GmbH"
        case 0x0A0E: return "Roland Corporation"
        case 0x0A0F: return "LIXIL Corporation"
        case 0x0A10: return "SUBARU Corporation"
        case 0x0A11: return "Sensolus"
        case 0x0A12: return "Dyson Technology Limited"
        case 0x0A13: return "Tec4med LifeScience GmbH"
        case 0x0A14: return "CROXEL, INC."
        case 0x0A15: return "Syng Inc"
        case 0x0A16: return "RIDE VISION LTD"
        case 0x0A17: return "Plume Design Inc"
        case 0x0A18: return "Cambridge Animal Technologies Ltd"
        case 0x0A19: return "Maxell, Ltd."
        case 0x0A1A: return "Link Labs, Inc."
        case 0x0A1B: return "Embrava Pty Ltd"
        case 0x0A1C: return "INPEAK S.C."
        case 0x0A1D: return "API-K"
        case 0x0A1E: return "CombiQ AB"
        case 0x0A1F: return "DeVilbiss Healthcare LLC"
        case 0x0A20: return "Jiangxi Innotech Technology Co., Ltd"
        case 0x0A21: return "Apollogic Sp. z o.o."
        case 0x0A22: return "DAIICHIKOSHO CO., LTD."
        case 0x0A23: return "BIXOLON CO.,LTD"
        case 0x0A24: return "Atmosic Technologies, Inc."
        case 0x0A25: return "Eran Financial Services LLC"
        case 0x0A26: return "Louis Vuitton"
        case 0x0A27: return "AYU DEVICES PRIVATE LIMITED"
        case 0x0A28: return "NanoFlex"
        case 0x0A29: return "Worthcloud Technology Co.,Ltd"
        case 0x0A2A: return "Yamaha Corporation"
        case 0x0A2B: return "PaceBait IVS"
        case 0x0A2C: return "Shenzhen H&T Intelligent Control Co., Ltd"
        case 0x0A2D: return "Shenzhen Feasycom Technology Co., Ltd."
        case 0x0A2E: return "Zuma Array Limited"
        case 0x0A2F: return "Instamic, Inc."
        case 0x0A30: return "Air-Weigh"
        case 0x0A31: return "Nevro Corp."
        case 0x0A32: return "Pinnacle Technology, Inc."
        case 0x0A33: return "WMF AG"
        case 0x0A34: return "Luxer Corporation"
        case 0x0A35: return "safectory GmbH"
        case 0x0A36: return "NGK SPARK PLUG CO., LTD."
        case 0x0A37: return "2587702 Ontario Inc."
        case 0x0A38: return "Bouffalo Lab (Nanjing)., Ltd."
        case 0x0A39: return "BLUETICKETING SRL"
        case 0x0A3A: return "Incotex Co. Ltd."
        case 0x0A3B: return "Galileo Technology Limited"
        case 0x0A3C: return "Siteco GmbH"
        case 0x0A3D: return "DELABIE"
        case 0x0A3E: return "Hefei Yunlian Semiconductor Co., Ltd"
        case 0x0A3F: return "Shenzhen Yopeak Optoelectronics Technology Co., Ltd."
        case 0x0A40: return "GEWISS S.p.A."
        case 0x0A41: return "OPEX Corporation"
        case 0x0A42: return "Motionalysis, Inc."
        case 0x0A43: return "Busch Systems International Inc."
        case 0x0A44: return "Novidan, Inc."
        case 0x0A45: return "3SI Security Systems, Inc"
        case 0x0A46: return "Beijing HC-Infinite Technology Limited"
        case 0x0A47: return "The Wand Company Ltd"
        case 0x0A48: return "JRC Mobility Inc."
        case 0x0A49: return "Venture Research Inc."
        case 0x0A4A: return "Map Large, Inc."
        case 0x0A4B: return "MistyWest Energy and Transport Ltd."
        case 0x0A4C: return "SiFli Technologies (shanghai) Inc."
        case 0x0A4D: return "Lockn Technologies Private Limited"
        case 0x0A4E: return "Toytec Corporation"
        case 0x0A4F: return "VANMOOF Global Holding B.V."
        case 0x0A50: return "Nextscape Inc."
        case 0x0A51: return "CSIRO"
        case 0x0A52: return "Follow Sense Europe B.V."
        case 0x0A53: return "KKM COMPANY LIMITED"
        case 0x0A54: return "SQL Technologies Corp."
        case 0x0A55: return "Inugo Systems Limited"
        case 0x0A56: return "ambie"
        case 0x0A57: return "Meizhou Guo Wei Electronics Co., Ltd"
        case 0x0A58: return "Indigo Diabetes"
        case 0x0A59: return "TourBuilt, LLC"
        case 0x0A5A: return "Sontheim Industrie Elektronik GmbH"
        case 0x0A5B: return "LEGIC Identsystems AG"
        case 0x0A5C: return "Innovative Design Labs Inc."
        case 0x0A5D: return "MG Energy Systems B.V."
        case 0x0A5E: return "LaceClips llc"
        case 0x0A5F: return "stryker"
        case 0x0A60: return "DATANG SEMICONDUCTOR TECHNOLOGY CO.,LTD"
        case 0x0A61: return "Smart Parks B.V."
        case 0x0A62: return "MOKO TECHNOLOGY Ltd"
        case 0x0A63: return "Gremsy JSC"
        case 0x0A64: return "Geopal system A/S"
        case 0x0A65: return "Lytx, INC."
        case 0x0A66: return "JUSTMORPH PTE. LTD."
        case 0x0A67: return "Beijing SuperHexa Century Technology CO. Ltd"
        case 0x0A68: return "Focus Ingenieria SRL"
        case 0x0A69: return "HAPPIEST BABY, INC."
        case 0x0A6A: return "Scribble Design Inc."
        case 0x0A6B: return "Olympic Ophthalmics, Inc."
        case 0x0A6C: return "Pokkels"
        case 0x0A6D: return "KUUKANJYOKIN Co.,Ltd."
        case 0x0A6E: return "Pac Sane Limited"
        case 0x0A6F: return "Warner Bros."
        case 0x0A70: return "Ooma"
        case 0x0A71: return "Senquip Pty Ltd"
        case 0x0A72: return "Jumo GmbH & Co. KG"
        case 0x0A73: return "Innohome Oy"
        case 0x0A74: return "MICROSON S.A."
        case 0x0A75: return "Delta Cycle Corporation"
        case 0x0A76: return "Synaptics Incorporated"
        case 0x0A77: return "JMD PACIFIC PTE. LTD."
        case 0x0A78: return "Shenzhen Sunricher Technology Limited"
        case 0x0A79: return "Webasto SE"
        case 0x0A7A: return "Emlid Limited"
        case 0x0A7B: return "UniqAir Oy"
        case 0x0A7C: return "WAFERLOCK"
        case 0x0A7D: return "Freedman Electronics Pty Ltd"
        case 0x0A7E: return "KEBA Handover Automation GmbH"
        case 0x0A7F: return "Intuity Medical"
        case 0x0A80: return "Cleer Limited"
        case 0x0A81: return "Universal Biosensors Pty Ltd"
        case 0x0A82: return "Corsair"
        case 0x0A83: return "Rivata, Inc."
        case 0x0A84: return "Greennote Inc,"
        case 0x0A85: return "Snowball Technology Co., Ltd."
        case 0x0A86: return "ALIZENT International"
        case 0x0A87: return "Shanghai Smart System Technology Co., Ltd"
        case 0x0A88: return "PSA Peugeot Citroen"
        case 0x0A89: return "SES-Imagotag"
        case 0x0A8A: return "HAINBUCH SPANNENDE TECHNIK"
        case 0x0A8B: return "SANlight GmbH"
        case 0x0A8C: return "DelpSys, s.r.o."
        case 0x0A8D: return "JCM TECHNOLOGIES S.A."
        case 0x0A8E: return "Perfect Company"
        case 0x0A8F: return "TOTO LTD."
        case 0x0A90: return "Shenzhen Grandsun Electronic Co.,Ltd."
        case 0x0A91: return "Monarch International Inc."
        case 0x0A92: return "Carestream Dental LLC"
        case 0x0A93: return "GiPStech S.r.l."
        case 0x0A94: return "OOBIK Inc."
        case 0x0A95: return "Pamex Inc."
        case 0x0A96: return "Lightricity Ltd"
        case 0x0A97: return "SensTek"
        case 0x0A98: return "Foil, Inc."
        case 0x0A99: return "Shanghai high-flying electronics technology Co.,Ltd"
        case 0x0A9A: return "TEMKIN ASSOCIATES, LLC"
        case 0x0A9B: return "Eello LLC"
        case 0x0A9C: return "Xi'an Fengyu Information Technology Co., Ltd."
        case 0x0A9D: return "Canon Finetech Nisca Inc."
        case 0x0A9E: return "LifePlus, Inc."
        case 0x0A9F: return "ista International GmbH"
        case 0x0AA0: return "Loy Tec electronics GmbH"
        case 0x0AA1: return "LINCOGN TECHNOLOGY CO. LIMITED"
        case 0x0AA2: return "Care Bloom, LLC"
        case 0x0AA3: return "DIC Corporation"
        case 0x0AA4: return "FAZEPRO LLC"
        case 0x0AA5: return "Shenzhen Uascent Technology Co., Ltd"
        case 0x0AA6: return "Realityworks, inc."
        case 0x0AA7: return "Urbanista AB"
        case 0x0AA8: return "Zencontrol Pty Ltd"
        case 0x0AA9: return "Mrinq Technologies LLC"
        case 0x0AAA: return "Computime International Ltd"
        case 0x0AAB: return "Anhui Listenai Co"
        case 0x0AAC: return "OSM HK Limited"
        case 0x0AAD: return "Adevo Consulting AB"
        case 0x0AAE: return "PS Engineering, Inc."
        case 0x0AAF: return "AIAIAI ApS"
        case 0x0AB0: return "Visiontronic s.r.o."
        case 0x0AB1: return "InVue Security Products Inc"
        case 0x0AB2: return "TouchTronics, Inc."
        case 0x0AB3: return "INNER RANGE PTY. LTD."
        case 0x0AB4: return "Ellenby Technologies, Inc."
        case 0x0AB5: return "Elstat Ltd [ Formerly Elstat Electronics Ltd.]"
        case 0x0AB6: return "Xenter, Inc."
        case 0x0AB7: return "LogTag North America Inc."
        case 0x0AB8: return "Sens.ai Incorporated"
        case 0x0AB9: return "STL"
        case 0x0ABA: return "Open Bionics Ltd."
        case 0x0ABB: return "R-DAS, s.r.o."
        case 0x0ABC: return "KCCS Mobile Engineering Co., Ltd."
        case 0x0ABD: return "Inventas AS"
        case 0x0ABE: return "Robkoo Information & Technologies Co., Ltd."
        case 0x0ABF: return "PAUL HARTMANN AG"
        case 0x0AC0: return "Omni-ID USA, INC."
        case 0x0AC1: return "Shenzhen Jingxun Technology Co., Ltd."
        case 0x0AC2: return "RealMega Microelectronics technology (Shanghai) Co. Ltd."
        case 0x0AC3: return "Kenzen, Inc."
        case 0x0AC4: return "CODIUM"
        case 0x0AC5: return "Flexoptix GmbH"
        case 0x0AC6: return "Barnes Group Inc."
        case 0x0AC7: return "Chengdu Aich Technology Co.,Ltd"
        case 0x0AC8: return "Keepin Co., Ltd."
        case 0x0AC9: return "Swedlock AB"
        case 0x0ACA: return "Shenzhen CoolKit Technology Co., Ltd"
        case 0x0ACB: return "ise Individuelle Software und Elektronik GmbH"
        case 0x0ACC: return "Nuvoton"
        case 0x0ACD: return "Visuallex Sport International Limited"
        case 0x0ACE: return "KOBATA GAUGE MFG. CO., LTD."
        case 0x0ACF: return "CACI Technologies"
        case 0x0AD0: return "Nordic Strong ApS"
        case 0x0AD1: return "EAGLE KINGDOM TECHNOLOGIES LIMITED"
        case 0x0AD2: return "Lautsprecher Teufel GmbH"
        case 0x0AD3: return "SSV Software Systems GmbH"
        case 0x0AD4: return "Zhuhai Pantum Electronisc Co., Ltd"
        case 0x0AD5: return "Streamit B.V."
        case 0x0AD6: return "nymea GmbH"
        case 0x0AD7: return "AL-KO Geraete GmbH"
        case 0x0AD8: return "Franz Kaldewei GmbH&Co KG"
        case 0x0AD9: return "Shenzhen Aimore. Co.,Ltd"
        case 0x0ADA: return "Codefabrik GmbH"
        case 0x0ADB: return "Reelables, Inc."
        case 0x0ADC: return "Duravit AG "
        case 0x0ADD: return "Boss Audio"
        case 0x0ADE: return "Vocera Communications, Inc."
        case 0x0ADF: return "Douglas Dynamics L.L.C."
        case 0x0AE0: return "Viceroy Devices Corporation"
        case 0x0AE1: return "ChengDu ForThink Technology Co., Ltd."
        case 0x0AE2: return "IMATRIX SYSTEMS, INC."
        case 0x0AE3: return "GlobalMed"
        case 0x0AE4: return "DALI Alliance"
        case 0x0AE5: return "unu GmbH"
        case 0x0AE6: return "Hexology"
        case 0x0AE7: return "Sunplus Technology Co., Ltd."
        case 0x0AE8: return "LEVEL, s.r.o."
        case 0x0AE9: return "FLIR Systems AB"
        case 0x0AEA: return "Borda Technology"
        case 0x0AEB: return "Square, Inc."
        case 0x0AEC: return "FUTEK ADVANCED SENSOR TECHNOLOGY, INC"
        case 0x0AED: return "Saxonar GmbH"
        case 0x0AEE: return "Velentium, LLC"
        case 0x0AEF: return "GLP German Light Products GmbH"
        case 0x0AF0: return "Leupold & Stevens, Inc."
        case 0x0AF1: return "CRADERS,CO.,LTD"
        case 0x0AF2: return "Shanghai All Link Microelectronics Co.,Ltd"
        case 0x0AF3: return "701x Inc."
        case 0x0AF4: return "Radioworks Microelectronics PTY LTD"
        case 0x0AF5: return "Unitech Electronic Inc."
        case 0x0AF6: return "AMETEK, Inc."
        case 0x0AF7: return "Irdeto"
        case 0x0AF8: return "First Design System Inc."
        case 0x0AF9: return "Unisto AG"
        case 0x0AFA: return "Chengdu Ambit Technology Co., Ltd."
        case 0x0AFB: return "SMT ELEKTRONIK GmbH"
        case 0x0AFC: return "Cerebrum Sensor Technologies Inc."
        case 0x0AFD: return "Weber Sensors, LLC"
        case 0x0AFE: return "Earda Technologies Co.,Ltd"
        case 0x0AFF: return "FUSEAWARE LIMITED"
        case 0x0B00: return "Flaircomm Microelectronics Inc."
        case 0x0B01: return "RESIDEO TECHNOLOGIES, INC."
        case 0x0B02: return "IORA Technology Development Ltd. Sti."
        case 0x0B03: return "Precision Triathlon Systems Limited"
        case 0x0B04: return "I-PERCUT"
        case 0x0B05: return "Marquardt GmbH"
        case 0x0B06: return "FAZUA GmbH"
        case 0x0B07: return "Workaround Gmbh"
        case 0x0B08: return "Shenzhen Qianfenyi Intelligent Technology Co., LTD"
        case 0x0B09: return "soonisys"
        case 0x0B0A: return "Belun Technology Company Limited"
        case 0x0B0B: return "Sanistaal A/S"
        case 0x0B0C: return "BluPeak"
        case 0x0B0D: return "SANYO DENKO Co.,Ltd."
        case 0x0B0E: return "Honda Lock Mfg. Co.,Ltd."
        case 0x0B0F: return "B.E.A. S.A."
        case 0x0B10: return "Alfa Laval Corporate AB"
        case 0x0B11: return "ThermoWorks, Inc."
        case 0x0B12: return "ToughBuilt Industries LLC"
        case 0x0B13: return "IOTOOLS"
        case 0x0B14: return "Olumee"
        case 0x0B15: return "NAOS JAPAN K.K."
        case 0x0B16: return "Guard RFID Solutions Inc."
        case 0x0B17: return "SIG SAUER, INC."
        case 0x0B18: return "DECATHLON SE"
        case 0x0B19: return "WBS PROJECT H PTY LTD"
        case 0x0B1A: return "Roca Sanitario, S.A."
        case 0x0B1B: return "Enerpac Tool Group Corp."
        case 0x0B1C: return "Nanoleq AG"
        case 0x0B1D: return "Accelerated Systems"
        case 0x0B1E: return "PB INC."
        case 0x0B1F: return "Beijing ESWIN Computing Technology Co., Ltd."
        case 0x0B20: return "TKH Security B.V."
        case 0x0B21: return "ams AG"
        case 0x0B22: return "Hygiene IQ, LLC."
        case 0x0B23: return "iRhythm Technologies, Inc."
        case 0x0B24: return "BeiJing ZiJie TiaoDong KeJi Co.,Ltd."
        case 0x0B25: return "NIBROTECH LTD"
        case 0x0B26: return "Baracoda Daily Healthtech."
        case 0x0B27: return "Lumi United Technology Co., Ltd"
        case 0x0B28: return "CHACON"
        case 0x0B29: return "Tech-Venom Entertainment Private Limited"
        case 0x0B2A: return "ACL Airshop B.V."
        case 0x0B2B: return "MAI"
        case 0x0B2C: return "ILLUMAGEAR, Inc."
        case 0x0B2D: return "REDARC ELECTRONICS PTY LTD"
        case 0x0B2E: return "MOCA System Inc."
        case 0x0B2F: return "Duke Manufacturing Co"
        case 0x0B30: return "ART SPA"
        case 0x0B31: return "Silver Wolf Vehicles Inc."
        case 0x0B32: return "Hala Systems, Inc."
        case 0x0B33: return "ARMATURA LLC"
        case 0x0B34: return "CONZUMEX INDUSTRIES PRIVATE LIMITED"
        case 0x0B35: return "BH Sens"
        case 0x0B36: return "SINTEF"
        case 0x0B37: return "Omnivoltaic Energy Solutions Limited Company"
        case 0x0B38: return "WISYCOM S.R.L."
        case 0x0B39: return "Red 100 Lighting Co., ltd."
        case 0x0B3A: return "Impact Biosystems, Inc."
        case 0x0B3B: return "AIC semiconductor (Shanghai) Co., Ltd."
        case 0x0B3C: return "Dodge Industrial, Inc."
        case 0x0B3D: return "REALTIMEID AS"
        case 0x0B3E: return "ISEO Serrature S.p.a."
        case 0x0B3F: return "MindRhythm, Inc."
        case 0x0B40: return "Havells India Limited"
        case 0x0B41: return "Sentrax GmbH"
        case 0x0B42: return "TSI"
        case 0x0B43: return "INCITAT ENVIRONNEMENT"
        case 0x0B44: return "nFore Technology Co., Ltd."
        case 0x0B45: return "Electronic Sensors, Inc."
        case 0x0B46: return "Bird Rides, Inc."
        case 0x0B47: return "Gentex Corporation "
        case 0x0B48: return "NIO USA, Inc."
        case 0x0B49: return "SkyHawke Technologies"
        case 0x0B4A: return "Nomono AS"
        case 0x0B4B: return "EMS Integrators, LLC"
        case 0x0B4C: return "BiosBob.Biz"
        case 0x0B4D: return "Adam Hall GmbH"
        case 0x0B4E: return "ICP Systems B.V."
        case 0x0B4F: return "Breezi.io, Inc."
        case 0x0B50: return "Mesh Systems LLC"
        case 0x0B51: return "FUN FACTORY GmbH"
        case 0x0B52: return "ZIIP Inc"
        case 0x0B53: return "SHENZHEN KAADAS INTELLIGENT TECHNOLOGY CO.,Ltd"
        case 0x0B54: return "Emotion Fitness GmbH & Co. KG"
        case 0x0B55: return "H G M Automotive Electronics, Inc."
        case 0x0B56: return "BORA - Vertriebs GmbH & Co KG"
        case 0x0B57: return "CONVERTRONIX TECHNOLOGIES AND SERVICES LLP"
        case 0x0B58: return "TOKAI-DENSHI INC"
        case 0x0B59: return "Versa Group B.V."
        case 0x0B5A: return "H.P. Shelby Manufacturing, LLC."
        case 0x0B5B: return "Shenzhen ImagineVision Technology Limited"
        case 0x0B5C: return "Exponential Power, Inc."
        case 0x0B5D: return "Fujian Newland Auto-ID Tech. Co., Ltd."
        case 0x0B5E: return "CELLCONTROL, INC."
        case 0x0B5F: return "Rivieh, Inc."
        case 0x0B60: return "RATOC Systems, Inc."
        case 0x0B61: return "Sentek Pty Ltd"
        case 0x0B62: return "NOVEA ENERGIES"
        // Last update date: 22.04.2022
            
        case 0xFFFF: return "Unassigned"
        default:     return nil
        }
    }
    
}
